from __future__ import annotations

import os
import subprocess
from pathlib import Path
from unittest.mock import patch

import pytest

from auditwheel.error import InvalidLibcError
from auditwheel.libc import Libc, LibcVersion, _find_musl_libc, _get_musl_version


@patch("auditwheel.libc.Path")
def test_find_musllinux_not_found(path_mock):
    path_mock.return_value.glob.return_value = []
    with pytest.raises(InvalidLibcError):
        _find_musl_libc()
    assert Libc.detect() != Libc.MUSL


@patch("auditwheel.libc.Path")
def test_find_musllinux_found(path_mock):
    path_mock.return_value.glob.return_value = ["/lib/ld-musl-dummy.so.1"]
    musl = _find_musl_libc()
    assert str(musl) == "/lib/ld-musl-dummy.so.1"
    assert Libc.detect() == Libc.MUSL


def test_get_musl_version_invalid_path():
    with pytest.raises(InvalidLibcError):
        _get_musl_version(Path("/tmp/no/executable/here"))  # noqa: S108


@patch("auditwheel.libc.subprocess.run")
def test_get_musl_version_invalid_version(run_mock):
    run_mock.return_value = subprocess.CompletedProcess([], 1, None, "Version 1.1")
    with pytest.raises(InvalidLibcError):
        _get_musl_version(Path("anything"))


@patch("auditwheel.libc.subprocess.run")
def test_get_musl_version_valid_version(run_mock):
    run_mock.return_value = subprocess.CompletedProcess([], 1, None, "Version 5.6.7")
    version = _get_musl_version(Path("anything"))
    assert version.major == 5
    assert version.minor == 6


@patch("auditwheel.libc.Path")
def test_detect_glibc(path_mock):
    path_mock.return_value.glob.return_value = []
    assert Libc.detect() == Libc.GLIBC


@pytest.mark.parametrize(
    "confstr",
    [
        "glibc 42.42",
        "glibc 42.42-test",
        "glibc 42.42.0",
        "glibc 42.42~0",
    ],
)
def test_glibc_version(monkeypatch, confstr):
    monkeypatch.setattr(os, "confstr", lambda _: confstr)
    assert Libc.GLIBC.get_current_version() == LibcVersion(42, 42)


@pytest.mark.parametrize(
    "confstr",
    [
        None,
        "glibc",
        "glibc 42.42 test",
        "glibc 42",
        "glibc 42.test",
        "glibc test.42",
    ],
)
def test_bad_glibc_version(monkeypatch, confstr):
    monkeypatch.setattr(os, "confstr", lambda _: confstr)
    with pytest.raises(InvalidLibcError):
        Libc.GLIBC.get_current_version()


# Android is cross-compiled, so autodetection is not possible.
def test_android_version():
    with pytest.raises(InvalidLibcError, match=r"can't determine version of libc 'android'"):
        Libc.ANDROID.get_current_version()


@pytest.mark.parametrize(
    ("libc", "expected"),
    [
        (Libc.GLIBC, "manylinux"),
        (Libc.MUSL, "musllinux"),
        (Libc.ANDROID, "android"),
    ],
)
def test_tag_prefix(libc, expected):
    assert libc.tag_prefix == expected



from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any

from auditwheel import options

if TYPE_CHECKING:
    import argparse

logger = logging.getLogger(__name__)


def configure_subparser(sub_parsers: Any) -> None:  # noqa: ANN401
    help_ = "Analyze a single ELF file (similar to ``ldd``)."
    p = sub_parsers.add_parser("lddtree", help=help_, description=help_)
    p.add_argument("file", type=Path, help="Path to .so file")
    options.ldpaths(p)
    p.set_defaults(func=execute)


def execute(args: argparse.Namespace, p: argparse.ArgumentParser) -> int:  # noqa: ARG001
    from auditwheel import json
    from auditwheel.lddtree import ld_paths_from_arg, ldd

    logger.info(json.dumps(ldd(args.file, ldpaths=ld_paths_from_arg(args.LDPATHS))))
    return 0



# SPDX-License-Identifier: MIT

"""
Module which provides (heh) 'yum provides'
functionality across many package managers.
"""

import dataclasses
import pathlib
import re
import shutil
import subprocess
import sys
import typing
from urllib.parse import quote

__all__ = ["ProvidedBy", "whichprovides"]
__version__ = "0.4.0"

_OS_RELEASE_LINES_RE = re.compile(r"^([A-Z_]+)=(?:\"([^\"]*)\"|(.*))$", re.MULTILINE)
_APK_WHO_OWNS_RE = re.compile(r" is owned by ([^\s\-]+)-([^\s]+)$", re.MULTILINE)
_DPKG_SEARCH_RE = re.compile(r"^([^:]+):")
_DPKG_VERSION_RE = re.compile(r"^Version: ([^\s]+)", re.MULTILINE)
_APT_FILE_SEARCH_RE = re.compile(r"^([^:]+): (.+)$", re.MULTILINE)


@dataclasses.dataclass
class ProvidedBy:
    package_type: str
    package_name: str
    package_version: str
    distro: typing.Union[str, None] = None

    @property
    def purl(self) -> str:
        """The Package URL (PURL) of the providing package"""

        def _quote_purl(value: str) -> str:
            """
            Quotes according to PURL rules which are different from
            typical URL percent encoding.
            """
            return quote(value, safe="")

        # PURL disallows many characters in the package type field.
        if not re.match(r"^[a-zA-Z0-9\+\-\.]+$", self.package_type):
            raise ValueError("Package type must be ASCII letters, numbers, +, -, and .")

        parts = ["pkg:", self.package_type.lower(), "/"]
        if self.distro:
            parts.extend((_quote_purl(self.distro), "/"))
        parts.extend(
            (_quote_purl(self.package_name), "@", _quote_purl(self.package_version))
        )
        return "".join(parts)


class PackageProvider:
    # Order in which the provider should be resolved.
    # Lower is attempted earlier than higher numbers.
    _resolve_order: int = 0
    _has_bin_cache: dict[str, typing.Union[str, bool]] = {}

    @staticmethod
    def os_release() -> dict[str, str]:
        """Dumb method of finding os-release information."""
        try:
            with open("/etc/os-release") as f:
                os_release = {}
                for name, value_quoted, value_unquoted in _OS_RELEASE_LINES_RE.findall(
                    f.read()
                ):
                    value = value_quoted if value_quoted else value_unquoted
                    os_release[name] = value
                return os_release
        except OSError:
            return {}

    @staticmethod
    def distro() -> typing.Optional[str]:
        return PackageProvider.os_release().get("ID", None)

    @classmethod
    def which(
        cls, bin: str, *, allowed_returncodes: typing.Optional[set[int]] = None
    ) -> typing.Optional[str]:
        """which, but tries to execute the program, too!"""
        cached_bin = cls._has_bin_cache.get(bin)
        assert cached_bin is not True
        if cached_bin is False:
            return None
        if cached_bin is not None:
            return cached_bin
        bin_which = shutil.which(bin)
        if bin_which is None:  # Cache the 'not-found' result.
            cls._has_bin_cache[bin] = False
            return None
        try:
            subprocess.check_call(
                [bin_which, "--version"],
                stdout=subprocess.DEVNULL,
                stderr=subprocess.DEVNULL,
            )
            cls._has_bin_cache[bin] = bin_which
            return bin_which
        except subprocess.CalledProcessError as e:
            # If running --version returns an non-zero exit we
            # explicitly allow that here.
            if allowed_returncodes and e.returncode in allowed_returncodes:
                cls._has_bin_cache[bin] = bin_which
                return bin_which
            cls._has_bin_cache[bin] = False
        return None

    @classmethod
    def is_available(cls) -> bool:
        return False

    @classmethod
    def whichprovides(cls, filepaths: typing.Collection[str]) -> dict[str, ProvidedBy]:
        raise NotImplementedError()


class _SinglePackageProvider(PackageProvider):
    """Abstract PackageProvider for single-filepath APIs"""

    @classmethod
    def whichprovides(cls, filepaths: typing.Collection[str]) -> dict[str, ProvidedBy]:
        results = {}
        for filepath in filepaths:
            if provided_by := cls.whichprovides1(filepath):
                results[filepath] = provided_by
        return results

    @classmethod
    def whichprovides1(cls, filepath: str) -> typing.Optional[ProvidedBy]:
        raise NotImplementedError()


class ApkPackageProvider(_SinglePackageProvider):
    @classmethod
    def is_available(cls) -> bool:
        return bool(cls.which("apk") and cls.distro())

    @classmethod
    def whichprovides1(cls, filepath: str) -> typing.Optional[ProvidedBy]:
        apk_bin = cls.which("apk")
        distro = cls.distro()
        assert apk_bin is not None and distro is not None
        try:
            # $ apk info --who-owns /bin/bash
            # /bin/bash is owned by bash-5.2.26-r0
            stdout = subprocess.check_output(
                [apk_bin, "info", "--who-owns", str(filepath)],
                stderr=subprocess.DEVNULL,
            ).decode()
            if match := _APK_WHO_OWNS_RE.search(stdout):
                return ProvidedBy(
                    package_type="apk",
                    distro=distro,
                    package_name=match.group(1),
                    package_version=match.group(2),
                )
        except subprocess.CalledProcessError:
            pass
        return None


class RpmPackageProvider(_SinglePackageProvider):
    @classmethod
    def is_available(cls) -> bool:
        return bool(cls.which("rpm") and cls.distro())

    @classmethod
    def whichprovides1(cls, filepath: str) -> typing.Optional[ProvidedBy]:
        rpm_bin = cls.which("rpm")
        distro = cls.distro()
        assert rpm_bin is not None and distro is not None
        try:
            # $ rpm -qf --queryformat "%{NAME} %{VERSION} %{RELEASE} ${ARCH}" /bin/bash
            # bash 4.4.20 4.el8_6
            stdout = subprocess.check_output(
                [
                    rpm_bin,
                    "-qf",
                    "--queryformat",
                    "%{NAME} %{VERSION} %{RELEASE} %{ARCH}",
                    str(filepath),
                ],
                stderr=subprocess.DEVNULL,
            ).decode()
            package_name, package_version, package_release, *_ = stdout.strip().split(
                " ", 4
            )
            return ProvidedBy(
                package_type="rpm",
                distro=distro,
                package_name=package_name,
                package_version=f"{package_version}-{package_release}",
            )
        except subprocess.CalledProcessError:
            pass
        return None


class DpkgPackageProvider(_SinglePackageProvider):
    @classmethod
    def is_available(cls) -> bool:
        return bool(cls.which("dpkg") and cls.distro())

    @classmethod
    def whichprovides1(cls, filepath: str) -> typing.Optional[ProvidedBy]:
        dpkg_bin = cls.which("dpkg")
        distro = cls.distro()
        assert dpkg_bin is not None and distro is not None
        try:
            # $ dpkg -S /bin/bash
            # bash: /bin/bash
            stdout = subprocess.check_output(
                [dpkg_bin, "-S", str(filepath)],
                stderr=subprocess.DEVNULL,
            ).decode()
            if match := _DPKG_SEARCH_RE.search(stdout):
                package_name = match.group(1)
                # $ dpkg -s bash
                # ...
                # Version: 5.1-6ubuntu1.1
                stdout = subprocess.check_output(
                    [dpkg_bin, "-s", package_name],
                    stderr=subprocess.DEVNULL,
                ).decode()
                if match := _DPKG_VERSION_RE.search(stdout):
                    return ProvidedBy(
                        package_type="deb",
                        distro=distro,
                        package_name=package_name,
                        package_version=match.group(1),
                    )
        except subprocess.CalledProcessError:
            pass
        return None


class AptFilePackageProvider(PackageProvider):
    # apt-file is slow, so resolve this one later.
    _resolve_order = 100

    @classmethod
    def is_available(cls) -> bool:
        return bool(
            cls.which("apt")
            and cls.which("apt-file", allowed_returncodes={2})
            and cls.distro()
        )

    @classmethod
    def whichprovides(cls, filepaths: typing.Collection[str]) -> dict[str, ProvidedBy]:
        apt_bin = cls.which("apt")
        apt_file_bin = cls.which("apt-file", allowed_returncodes={2})
        distro = cls.distro()
        assert apt_bin is not None and apt_file_bin is not None and distro is not None
        results = {}
        try:
            # $ echo '\n'.join(paths) | apt-file search --from-file -
            # Finding relevant cache files to search ...
            # ...
            # libwebpdemux2: /usr/lib/x86_64-linux-gnu/libwebpdemux.so.2.0.9
            stdout = subprocess.check_output(
                [apt_file_bin, "search", "--from-file", "-"],
                stderr=subprocess.DEVNULL,
                input=b"\n".join(
                    [str(filepath).encode("utf-8") for filepath in filepaths]
                ),
            ).decode()
            for package_name, filepath in _APT_FILE_SEARCH_RE.findall(stdout):
                stdout = subprocess.check_output(
                    [apt_bin, "show", package_name],
                    stderr=subprocess.DEVNULL,
                ).decode()
                if match := _DPKG_VERSION_RE.search(stdout):
                    package_version = match.group(1)
                    results[filepath] = ProvidedBy(
                        package_type="deb",
                        distro=distro,
                        package_name=package_name,
                        package_version=package_version,
                    )
        except subprocess.CalledProcessError:
            pass
        return results


def _package_providers() -> list[type[PackageProvider]]:
    """Returns a list of package providers sorted in
    the order that they should be attempted.
    """

    def all_subclasses(cls):
        subclasses = set()
        for subcls in cls.__subclasses__():
            subclasses.add(subcls)
            subclasses |= all_subclasses(subcls)
        return subclasses

    return sorted(all_subclasses(PackageProvider), key=lambda p: p._resolve_order)


def whichprovides(filepath: typing.Union[str, list[str]]) -> dict[str, ProvidedBy]:
    """Return a package URL (PURL) for the package that provides a file"""
    if isinstance(filepath, str):
        filepaths = [filepath]
    else:
        filepaths = filepath

    # Link between the original path to the resolved
    # path and then allocate a structure for results.
    resolved_filepaths = {
        str(pathlib.Path(filepath).resolve()): filepath for filepath in filepaths
    }
    filepath_provided_by: dict[str, ProvidedBy] = {}
    for package_provider in _package_providers():
        remaining = set(resolved_filepaths) - set(filepath_provided_by)
        if not remaining:
            break
        if not package_provider.is_available():
            continue
        results = package_provider.whichprovides(remaining)
        filepath_provided_by.update(results)

    return {
        resolved_filepaths[filepath]: value
        for filepath, value in filepath_provided_by.items()
    }


def _main():
    if len(sys.argv) < 2:
        print(
            "Must provide one or more path argument "
            "('$ python -m whichprovides <paths>')",
            file=sys.stderr,
        )
        sys.exit(1)

    filepaths = sys.argv[1:]
    provided_bys = whichprovides(filepaths)
    exit_code = 0
    for filepath in filepaths:
        provided_by = provided_bys.get(filepath)
        if provided_by:
            print(f"{filepath}: {provided_by.purl}")
        else:
            print(f"No known package providing {filepath}", file=sys.stderr)
            exit_code = 1
    sys.exit(exit_code)


if __name__ == "__main__":
    _main()



from __future__ import annotations

import subprocess
from os import getenv, path

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext

define_macros = [("_GNU_SOURCE", None)]
libraries = []
library_dirs = []
extra_compile_args = []

if getenv("WITH_DEPENDENCY", "0") == "1":
    libraries.append("dependency")
    library_dirs.append(path.abspath(path.dirname(__file__)))
    define_macros.append(("WITH_DEPENDENCY", "1"))

if getenv("WITH_ARCH", "") != "":
    extra_compile_args.extend((f"-march={getenv('WITH_ARCH')}", "-mneeded"))

libraries.extend(["m", "c"])


class BuildExt(build_ext):
    def run(self) -> None:
        cflags = ("-shared", "-fPIC", "-D_GNU_SOURCE", *extra_compile_args)
        cmd = f"gcc {' '.join(cflags)} dependency.c -o libdependency.so -lm -lc"
        subprocess.check_call(cmd.split())
        super().run()


setup(
    name="testdependencies",
    version="0.0.1",
    cmdclass={"build_ext": BuildExt},
    ext_modules=[
        Extension(
            "testdependencies",
            sources=["testdependencies.c"],
            extra_compile_args=extra_compile_args,
            define_macros=define_macros,
            libraries=libraries,
            library_dirs=library_dirs,
        ),
    ],
)



from __future__ import annotations

import argparse
import logging
import shutil
import zlib
from pathlib import Path
from typing import Any

from auditwheel import options
from auditwheel.architecture import Architecture
from auditwheel.error import NonPlatformWheelError, WheelToolsError
from auditwheel.libc import Libc
from auditwheel.patcher import Patchelf
from auditwheel.policy import WheelPolicies
from auditwheel.tools import EnvironmentDefault
from auditwheel.wheeltools import get_wheel_architecture, get_wheel_libc

logger = logging.getLogger(__name__)


def configure_parser(sub_parsers: Any) -> None:  # noqa: ANN401
    policies = WheelPolicies(libc=Libc.detect(), arch=Architecture.detect())
    policy_names = [p.name for p in policies if p != policies.linux]
    policy_names += [alias for p in policies for alias in p.aliases]
    policy_names += ["auto"]
    epilog = """PLATFORMS:
These are the possible platform tags for this machine, as specified by PEP 600.
Note that old, pre-PEP 600 tags are still usable and are listed as aliases
below.
"""
    for p in policies:
        epilog += f"- {p.name}"
        if len(p.aliases) > 0:
            epilog += f" (aliased by {', '.join(p.aliases)})"
        epilog += "\n"
    help_ = """Vendor in external shared library dependencies of a wheel.
If multiple wheels are specified, an error processing one
wheel will abort processing of subsequent wheels.
"""
    parser = sub_parsers.add_parser(
        "repair",
        help=help_,
        description=help_,
        epilog=epilog,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument("WHEEL_FILE", type=Path, help="Path to wheel file.", nargs="+")
    parser.add_argument(
        "-z",
        "--zip-compression-level",
        action=EnvironmentDefault,
        metavar="ZIP_COMPRESSION_LEVEL",
        env="AUDITWHEEL_ZIP_COMPRESSION_LEVEL",
        dest="ZIP_COMPRESSION_LEVEL",
        type=int,
        help="Compress level to be used to create zip file.",
        choices=list(range(zlib.Z_NO_COMPRESSION, zlib.Z_BEST_COMPRESSION + 1)),
        default=zlib.Z_DEFAULT_COMPRESSION,
    )
    parser.add_argument(
        "--plat",
        action=EnvironmentDefault,
        metavar="PLATFORM",
        env="AUDITWHEEL_PLAT",
        dest="PLAT",
        help="Desired target platform. See the available platforms under the "
        'PLATFORMS section below. (default: "auto")',
        choices=policy_names,
        default="auto",
    )
    parser.add_argument(
        "-L",
        "--lib-sdir",
        dest="LIB_SDIR",
        help=('Subdirectory in packages to store copied libraries. (default: ".libs")'),
        default=".libs",
    )
    parser.add_argument(
        "-w",
        "--wheel-dir",
        dest="WHEEL_DIR",
        type=Path,
        help=('Directory to store delocated wheels (default: "wheelhouse/")'),
        default="wheelhouse/",
    )
    parser.add_argument(
        "--no-update-tags",
        dest="UPDATE_TAGS",
        action="store_false",
        help=(
            "Do not update the wheel filename tags and WHEEL info"
            " to match the repaired platform tag."
        ),
        default=True,
    )
    parser.add_argument(
        "--strip",
        dest="STRIP",
        action="store_true",
        help="Strip symbols in the resulting wheel",
        default=False,
    )
    parser.add_argument(
        "--exclude",
        dest="EXCLUDE",
        help="Exclude SONAME from grafting into the resulting wheel "
        "Please make sure wheel metadata reflects your dependencies. "
        "See https://github.com/pypa/auditwheel/pull/411#issuecomment-1500826281 "
        "(can be specified multiple times) "
        "(can contain wildcards, for example libfoo.so.*)",
        action="append",
        default=[],
    )
    parser.add_argument(
        "--only-plat",
        dest="ONLY_PLAT",
        action="store_true",
        help="Do not check for higher policy compatibility",
        default=False,
    )
    options.disable_isa_check(parser)
    options.allow_pure_python_wheel(parser)
    options.ldpaths(parser)

    parser.set_defaults(func=execute)


def execute(args: argparse.Namespace, parser: argparse.ArgumentParser) -> int:
    from auditwheel.repair import repair_wheel
    from auditwheel.wheel_abi import analyze_wheel_abi

    exclude: frozenset[str] = frozenset(args.EXCLUDE)
    wheel_dir: Path = args.WHEEL_DIR.absolute()
    wheel_files: list[Path] = args.WHEEL_FILE

    requested_architecture: Architecture | None = None

    plat_base: str = args.PLAT
    for a in Architecture:
        suffix = f"_{a.value}"
        if plat_base.endswith(suffix):
            plat_base = plat_base[: -len(suffix)]
            requested_architecture = a
            break

    for wheel_file in wheel_files:
        if not wheel_file.is_file():
            parser.error(f"cannot access {wheel_file}. No such file")

        wheel_filename = wheel_file.name
        arch = requested_architecture
        is_pure_python = False
        try:
            arch = get_wheel_architecture(wheel_filename)
            if requested_architecture is not None and requested_architecture != arch:
                msg = (
                    f"can't repair wheel {wheel_filename} with {arch.value} architecture to a "
                    f"wheel targeting {requested_architecture.value}"
                )
                parser.error(msg)
        except (WheelToolsError, NonPlatformWheelError) as e:
            logger.warning(
                "The architecture could not be deduced from the wheel filename",
            )
            is_pure_python = isinstance(e, NonPlatformWheelError)

        try:
            libc = get_wheel_libc(wheel_filename)
        except WheelToolsError:
            logger.debug("The libc could not be deduced from the wheel filename")
            libc = None

        for lc in Libc:
            if plat_base.startswith(lc.tag_prefix):
                if libc is None:
                    libc = lc
                if libc != lc:
                    msg = (
                        f"can't repair wheel {wheel_filename} with {libc.name} libc to a wheel "
                        f"targeting {lc.name}"
                    )
                    parser.error(msg)

        logger.info("Repairing %s", wheel_filename)

        if not wheel_dir.exists():
            wheel_dir.mkdir(parents=True)

        try:
            wheel_abi = analyze_wheel_abi(
                libc,
                arch,
                wheel_file,
                exclude,
                disable_isa_ext_check=args.DISABLE_ISA_EXT_CHECK,
                allow_graft=True,
                requested_policy_base_name=plat_base,
                args_ldpaths=args.LDPATHS,
            )
        except NonPlatformWheelError as e:
            logger.info(e.message)
            if is_pure_python and args.ALLOW_PURE_PY_WHEEL:
                dest_fname = wheel_dir / wheel_file.name
                if not dest_fname.is_file() or not dest_fname.samefile(wheel_file):
                    shutil.copy2(wheel_file, dest_fname)
                # process next wheel
                continue
            return 1

        policies = wheel_abi.policies
        if plat_base == "auto":
            if wheel_abi.overall_policy == policies.linux:
                # we're getting 'linux', override
                plat = policies.lowest.name
            else:
                plat = wheel_abi.overall_policy.name
        else:
            plat = f"{plat_base}_{policies.architecture.value}"
        requested_policy = policies.get_policy_by_name(plat)

        if requested_policy > wheel_abi.sym_policy:
            msg = (
                f'cannot repair "{wheel_file}" to "{plat}" ABI because of the '
                "presence of too-recent versioned symbols. You'll need to compile "
                "the wheel on an older toolchain."
            )
            parser.error(msg)

        if requested_policy > wheel_abi.ucs_policy:
            msg = (
                f'cannot repair "{wheel_file}" to "{plat}" ABI because it was '
                "compiled against a UCS2 build of Python. You'll need to compile "
                "the wheel against a wide-unicode build of Python."
            )
            parser.error(msg)

        if requested_policy > wheel_abi.blacklist_policy:
            msg = (
                f'cannot repair "{wheel_file}" to "{plat}" ABI because it '
                "depends on black-listed symbols."
            )
            parser.error(msg)

        if requested_policy > wheel_abi.machine_policy:
            msg = (
                f'cannot repair "{wheel_file}" to "{plat}" ABI because it '
                "depends on unsupported ISA extensions."
            )
            parser.error(msg)

        abis = [requested_policy.name, *requested_policy.aliases]
        if (not args.ONLY_PLAT) and requested_policy < wheel_abi.overall_policy:
            logger.info(
                (
                    "Wheel is eligible for a higher priority tag. "
                    "You requested %s but I have found this wheel is "
                    "eligible for %s."
                ),
                plat,
                wheel_abi.overall_policy.name,
            )
            abis = [
                wheel_abi.overall_policy.name,
                *wheel_abi.overall_policy.aliases,
                *abis,
            ]

        patcher = Patchelf(requested_policy.name)
        out_wheel = repair_wheel(
            wheel_abi,
            wheel_file,
            abis=abis,
            lib_sdir=args.LIB_SDIR,
            out_dir=wheel_dir,
            update_tags=args.UPDATE_TAGS,
            patcher=patcher,
            strip=args.STRIP,
            zip_compression_level=args.ZIP_COMPRESSION_LEVEL,
        )

        if out_wheel is not None:
            logger.info("\nFixed-up wheel written to %s", out_wheel)
    return 0



from __future__ import annotations

import functools
import platform
import struct
import sys
from enum import Enum


class Architecture(Enum):
    value: str

    aarch64 = "aarch64"
    arm64_v8a = "arm64_v8a"  # Android
    armv7l = "armv7l"
    i686 = "i686"
    loongarch64 = "loongarch64"
    ppc64 = "ppc64"
    ppc64le = "ppc64le"
    riscv64 = "riscv64"
    s390x = "s390x"
    x86_64 = "x86_64"
    x86_64_v2 = "x86_64_v2"
    x86_64_v3 = "x86_64_v3"
    x86_64_v4 = "x86_64_v4"

    def __str__(self) -> str:
        return self.value

    @property
    def baseline(self) -> Architecture:
        if self.value.startswith("arm64"):
            return Architecture.aarch64
        if self.value.startswith("x86_64"):
            return Architecture.x86_64
        return self

    @classmethod
    @functools.lru_cache(None)
    def _member_list(cls) -> list[Architecture]:
        return list(cls)

    def is_subset(self, other: Architecture) -> bool:
        if self.baseline != other.baseline:
            return False
        member_list = Architecture._member_list()
        return member_list.index(self) <= member_list.index(other)

    def is_superset(self, other: Architecture) -> bool:
        if self.baseline != other.baseline:
            return False
        return other.is_subset(self)

    @staticmethod
    def detect(*, bits: int | None = None) -> Architecture:
        machine = platform.machine()
        if sys.platform.startswith("win"):
            machine = {"AMD64": "x86_64", "ARM64": "aarch64", "x86": "i686"}.get(
                machine,
                machine,
            )
        elif sys.platform.startswith("darwin"):
            machine = {"arm64": "aarch64"}.get(machine, machine)

        if bits is None:
            # c.f. https://github.com/pypa/packaging/pull/711
            bits = 8 * struct.calcsize("P")

        if machine in {"x86_64", "i686"}:
            machine = {64: "x86_64", 32: "i686"}[bits]
        elif machine in {"aarch64", "armv8l"}:
            # use armv7l policy for 64-bit arm kernel in 32-bit mode (armv8l)
            machine = {64: "aarch64", 32: "armv7l"}[bits]

        return Architecture(machine)



from __future__ import annotations

import json
import re
from contextlib import nullcontext as does_not_raise
from pathlib import Path

import pytest

import auditwheel.policy
from auditwheel.architecture import Architecture
from auditwheel.lddtree import DynamicExecutable, DynamicLibrary, Platform
from auditwheel.libc import Libc
from auditwheel.policy import (
    Policy,
    WheelPolicies,
    _validate_pep600_compliance,
    get_replace_platforms,
)


def ids(x):
    if isinstance(x, Libc):
        return x.name
    if isinstance(x, does_not_raise):
        return "NoError"
    if hasattr(x, "expected_exception"):
        return x.expected_exception
    return None


def raises(exception, match=None, escape=True):
    if escape and match is not None:
        match = re.escape(match)
    return pytest.raises(exception, match=match)


@pytest.mark.parametrize(
    ("name", "expected"),
    [
        ("linux_aarch64", []),
        ("manylinux1_ppc64le", ["linux_ppc64le"]),
        ("manylinux2014_x86_64", ["linux_x86_64"]),
        ("manylinux_2_24_x86_64", ["linux_x86_64"]),
    ],
)
def test_replacement_platform(name, expected):
    assert get_replace_platforms(name) == expected


def test_pep600_compliance():
    _validate_pep600_compliance(
        [
            {
                "name": "manylinux1",
                "priority": 100,
                "symbol_versions": {
                    "i686": {"CXXABI": ["1.3"]},
                },
                "lib_whitelist": ["libgcc_s.so.1"],
            },
            {
                "name": "manylinux2010",
                "priority": 90,
                "symbol_versions": {
                    "i686": {"CXXABI": ["1.3", "1.3.1"]},
                },
                "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
            },
        ],
    )

    _validate_pep600_compliance(
        [
            {
                "name": "manylinux1",
                "priority": 100,
                "symbol_versions": {
                    "i686": {"CXXABI": ["1.3"]},
                    "x86_64": {"CXXABI": ["1.3"]},
                },
                "lib_whitelist": ["libgcc_s.so.1"],
            },
            {
                "name": "manylinux2010",
                "priority": 90,
                "symbol_versions": {
                    "i686": {"CXXABI": ["1.3", "1.3.1"]},
                },
                "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
            },
        ],
    )

    with pytest.raises(ValueError, match=r"manylinux2010_i686.*CXXABI.*1.3.2"):
        _validate_pep600_compliance(
            [
                {
                    "name": "manylinux1",
                    "priority": 100,
                    "symbol_versions": {
                        "i686": {"CXXABI": ["1.3", "1.3.2"]},
                    },
                    "lib_whitelist": ["libgcc_s.so.1"],
                },
                {
                    "name": "manylinux2010",
                    "priority": 90,
                    "symbol_versions": {
                        "i686": {"CXXABI": ["1.3", "1.3.1"]},
                    },
                    "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
                },
            ],
        )

    with pytest.raises(ValueError, match=r"manylinux2010.*libstdc\+\+\.so\.6"):
        _validate_pep600_compliance(
            [
                {
                    "name": "manylinux1",
                    "priority": 100,
                    "symbol_versions": {
                        "i686": {"CXXABI": ["1.3"]},
                    },
                    "lib_whitelist": ["libgcc_s.so.1", "libstdc++.so.6"],
                },
                {
                    "name": "manylinux2010",
                    "priority": 90,
                    "symbol_versions": {
                        "i686": {"CXXABI": ["1.3", "1.3.1"]},
                    },
                    "lib_whitelist": ["libgcc_s.so.1"],
                },
            ],
        )


class TestPolicyAccess:
    def test_get_by_name(self):
        arch = Architecture.detect()
        policies = WheelPolicies(libc=Libc.GLIBC, arch=arch)
        assert policies.get_policy_by_name(f"manylinux_2_41_{arch}").priority == 51
        assert policies.get_policy_by_name(f"manylinux_2_36_{arch}").priority == 56
        if arch == Architecture.loongarch64:
            return
        assert policies.get_policy_by_name(f"manylinux_2_31_{arch}").priority == 61
        if arch == Architecture.riscv64:
            return
        assert policies.get_policy_by_name(f"manylinux_2_27_{arch}").priority == 65
        assert policies.get_policy_by_name(f"manylinux_2_24_{arch}").priority == 70
        assert policies.get_policy_by_name(f"manylinux2014_{arch}").priority == 80
        assert policies.get_policy_by_name(f"manylinux_2_17_{arch}").priority == 80
        if arch not in {Architecture.x86_64, Architecture.i686}:
            return
        assert policies.get_policy_by_name(f"manylinux2010_{arch}").priority == 90
        assert policies.get_policy_by_name(f"manylinux_2_12_{arch}").priority == 90
        assert policies.get_policy_by_name(f"manylinux1_{arch}").priority == 100
        assert policies.get_policy_by_name(f"manylinux_2_5_{arch}").priority == 100

    def test_get_by_name_missing(self):
        policies = WheelPolicies(libc=Libc.GLIBC, arch=Architecture.x86_64)
        with pytest.raises(LookupError):
            policies.get_policy_by_name("nosuchpolicy")

    def test_get_by_name_duplicate(self):
        policies = WheelPolicies(libc=Libc.GLIBC, arch=Architecture.x86_64)
        policy = Policy("duplicate", (), 0, {}, frozenset(), {})
        policies._policies = [policy, policy]
        with pytest.raises(RuntimeError):
            policies.get_policy_by_name("duplicate")


class TestLddTreeExternalReferences:
    """Tests for lddtree_external_references."""

    @pytest.mark.parametrize("libc", [Libc.GLIBC, Libc.ANDROID])
    def test_filter_libs(self, libc):
        """Test the nested filter_libs function."""
        filtered_libs = [
            "ld-linux-x86_64.so.1",
            "ld64.so.1",
            "ld64.so.2",
        ]
        unfiltered_libs = ["libfoo.so.1.0", "libbar.so.999.999.999"]
        (filtered_libs if libc == Libc.ANDROID else unfiltered_libs).append("libpython3.13.so")
        libs = filtered_libs + unfiltered_libs

        lddtree = DynamicExecutable(
            interpreter=None,
            libc=libc,
            path="/path/to/lib",
            realpath=Path("/path/to/lib"),
            platform=Platform(
                "",
                64,
                True,
                "EM_X86_64",
                Architecture.x86_64,
                None,
                None,
            ),
            needed=tuple(libs),
            libraries={
                lib: DynamicLibrary(lib, f"/path/to/{lib}", Path(f"/path/to/{lib}")) for lib in libs
            },
            rpath=(),
            runpath=(),
        )
        policies = WheelPolicies(
            libc=libc,
            arch=Architecture.x86_64,
            wheel_fn=(
                Path("spam-0.1-py3-none-android_24_x86_64.whl") if libc == Libc.ANDROID else None
            ),
        )
        full_external_refs = policies.lddtree_external_references(
            lddtree,
            Path("/path/to/wheel"),
        )

        # Assert that each policy only has the unfiltered libs.
        for policy in full_external_refs:
            if policy.startswith("linux_"):
                assert set(full_external_refs[policy].libs) == set()
            else:
                assert set(full_external_refs[policy].libs) == set(unfiltered_libs)


@pytest.mark.parametrize(
    ("libc", "musl_policy", "arch", "exception"),
    [
        # valid
        (Libc.detect(), None, Architecture.detect(), does_not_raise()),
        (Libc.GLIBC, None, Architecture.x86_64, does_not_raise()),
        (Libc.MUSL, "musllinux_1_1", Architecture.x86_64, does_not_raise()),
        (Libc.GLIBC, None, Architecture.aarch64, does_not_raise()),
        # invalid
        (
            Libc.GLIBC,
            "musllinux_1_1",
            Architecture.x86_64,
            raises(ValueError, "'musl_policy' shall be None"),
        ),
        (
            Libc.MUSL,
            "manylinux_1_1",
            Architecture.x86_64,
            raises(ValueError, "Invalid 'musl_policy'"),
        ),
        (Libc.MUSL, "musllinux_5_1", Architecture.x86_64, raises(AssertionError)),
        (Libc.MUSL, None, Architecture.x86_64, does_not_raise()),
    ],
    ids=ids,
)
def test_wheel_policies_args(libc, musl_policy, arch, exception):
    with exception:
        policies = WheelPolicies(libc=libc, musl_policy=musl_policy, arch=arch)
        assert policies.libc == libc
        assert policies.architecture == arch
        if musl_policy is not None:
            assert policies._musl_policy == musl_policy
        elif libc == Libc.MUSL:
            assert policies._musl_policy == "musllinux_1_2"


def test_policy_checks_glibc():
    policies = WheelPolicies(libc=Libc.GLIBC, arch=Architecture.x86_64)

    policy = policies.versioned_symbols_policy({"some_library.so": {"GLIBC_2.17"}})
    assert policy > policies.linux
    policy = policies.versioned_symbols_policy({"some_library.so": {"GLIBC_999"}})
    assert policy == policies.linux
    policy = policies.versioned_symbols_policy({"some_library.so": {"OPENSSL_1_1_0"}})
    assert policy == policies.highest
    policy = policies.versioned_symbols_policy({"some_library.so": {"IAMALIBRARY"}})
    assert policy == policies.highest
    assert policies.linux < policies.lowest < policies.highest


@pytest.mark.parametrize(
    ("wheel_level", "policy_level"),
    [(21, 21), (22, 21), (23, 21), (24, 24), (25, 24), (26, 26), (27, 27), (28, 27)],
)
@pytest.mark.parametrize(("arch"), [Architecture.arm64_v8a, Architecture.x86_64])
def test_android(wheel_level, policy_level, arch):
    for policy_json in json.loads(
        (Path(auditwheel.policy.__file__).parent / "android-policy.json").read_text(),
    ):
        if policy_json["name"] == f"android_{policy_level}":
            whitelist = frozenset(policy_json["lib_whitelist"])
            break
    else:
        raise AssertionError(policy_level)

    assert "libc.so" in whitelist
    assert ("libcamera2ndk.so" in whitelist) == (policy_level >= 24)
    assert ("libaaudio.so" in whitelist) == (policy_level >= 26)
    assert ("libneuralnetworks.so" in whitelist) == (policy_level >= 27)

    platform = f"android_{wheel_level}_{arch}"
    policies = WheelPolicies(
        libc=Libc.ANDROID,
        arch=arch.baseline,
        wheel_fn=Path(f"foo-1.0-py3-none-{platform}.whl"),
    )
    assert len(list(policies)) == 2
    assert policies.linux.whitelist == frozenset()

    assert policies.lowest is policies.highest
    assert policies.lowest.whitelist == whitelist


def test_android_no_filename():
    with pytest.raises(ValueError, match="wheel_fn is required when selecting Android policies"):
        WheelPolicies(libc=Libc.ANDROID, arch=Architecture.x86_64)


@pytest.mark.parametrize(
    ("filename", "message"),
    [
        (
            "foo-1.0-py3-none-android_21_arm64_v8a.android_22_arm64_v8a.whl",
            "Android wheels must have exactly one platform tag, got "
            "['android_21_arm64_v8a', 'android_22_arm64_v8a']",
        ),
        (
            "foo-1.0-py3-none-android_20_x86_64.whl",
            "no Android policies match the platform tag 'android_20_x86_64'",
        ),
    ],
)
def test_android_error(filename, message):
    with pytest.raises(ValueError, match=re.escape(message)):
        WheelPolicies(libc=Libc.ANDROID, arch=Architecture.x86_64, wheel_fn=Path(filename))



from __future__ import annotations


class AuditwheelError(Exception):
    def __init__(self, msg: str) -> None:
        super().__init__(msg)

    @property
    def message(self) -> str:
        assert isinstance(self.args[0], str)  # noqa: S101
        return self.args[0]


class InvalidLibcError(AuditwheelError):
    pass


class WheelToolsError(AuditwheelError):
    pass


class NonPlatformWheelError(AuditwheelError):
    """No ELF binaries in the wheel"""

    def __init__(self, architecture: str | None, libraries: list[str] | None) -> None:
        if architecture is None or not libraries:
            msg = (
                "This does not look like a platform wheel, no ELF executable "
                "or shared library file (including compiled Python C extension) "
                "found in the wheel archive"
            )
        else:
            libraries_str = "\n\t".join(libraries)
            msg = (
                "Invalid binary wheel: no ELF executable or shared library file "
                "(including compiled Python C extension) with a "
                f"{architecture!r} architecure found. The following "
                f"ELF files were found:\n\t{libraries_str}\n"
            )
        super().__init__(msg)



import os

import pytest


@pytest.fixture(autouse=True, scope="session")
def clean_env() -> None:
    variables = ("AUDITWHEEL_PLAT", "AUDITWHEEL_ZIP_COMPRESSION_LEVEL")
    for var in variables:
        os.environ.pop(var, None)



import pytest

from auditwheel.genericpkgctx import InGenericPkgCtx


def test_unknown_extension(tmp_path):
    file = tmp_path / "foo.txt"
    file.touch()
    with pytest.raises(ValueError, match=r".*File formats supported.*"):
        InGenericPkgCtx(file)


def test_conda_outpath(tmp_path):
    file = tmp_path / "foo.tar.bz2"
    file.touch()
    out = tmp_path / "out.tar.bz2"
    with pytest.raises(NotImplementedError):
        InGenericPkgCtx(file, out)


@pytest.mark.parametrize("ext", ["whl", "tar.bz2"])
def test_valid_ext(tmp_path, ext):
    file = tmp_path / f"foo.{ext}"
    file.touch()
    InGenericPkgCtx(file)



from __future__ import annotations

import logging
from collections import defaultdict
from pathlib import Path

import pretend
import pytest

from auditwheel import wheel_abi
from auditwheel.architecture import Architecture
from auditwheel.lddtree import DynamicExecutable, DynamicLibrary, Platform
from auditwheel.libc import Libc
from auditwheel.policy import ExternalReference, WheelPolicies


class TestGetWheelElfdata:
    @pytest.mark.parametrize(
        ("filenames", "message"),
        [
            (
                # A single invalid file
                [Path("purelib") / "foo"],
                "Invalid binary wheel, found the following shared library/libraries in"
                " purelib folder:\n\tfoo\nThe wheel has to be platlib compliant in "
                "order to be repaired by auditwheel.",
            ),
            (
                # Multiple invalid files
                [Path("purelib") / "foo", Path("purelib") / "bar"],
                "Invalid binary wheel, found the following shared library/libraries in"
                " purelib folder:\n\tfoo\n\tbar\nThe wheel has to be platlib compliant"
                " in order to be repaired by auditwheel.",
            ),
        ],
    )
    def test_finds_shared_library_in_purelib(
        self,
        filenames: list[Path],
        message: str,
        monkeypatch: pytest.MonkeyPatch,
    ) -> None:
        entered_context = pretend.stub(iter_files=lambda: filenames)
        context = pretend.stub(
            __enter__=lambda: entered_context,
            __exit__=lambda *_: None,
        )

        monkeypatch.setattr(wheel_abi, "InGenericPkgCtx", pretend.stub(__call__=lambda _: context))
        monkeypatch.setattr(
            wheel_abi,
            "elf_is_python_extension",
            lambda fn, elf: (fn, elf),
        )
        monkeypatch.setattr(
            wheel_abi,
            "elf_file_filter",
            lambda fns: [(fn, pretend.stub()) for fn in fns],
        )

        with pytest.raises(RuntimeError) as exec_info:
            wheel_abi.get_wheel_elfdata(
                Libc.GLIBC,
                Architecture.x86_64,
                Path("/fakepath"),
                exclude=frozenset(),
                args_ldpaths="",
            )

        assert exec_info.value.args == (message,)


def test_get_symbol_policies() -> None:
    policies = WheelPolicies(libc=Libc.GLIBC, arch=Architecture.x86_64)
    versioned_symbols = defaultdict(set, {"libc.so.6": {"GLIBC_2.2.5"}})
    external_versioned_symbols = {
        "libmvec.so.1": {
            "libc.so.6": {"GLIBC_2.2.5"},
            "libm.so.6": {"GLIBC_2.15", "GLIBC_2.2.5"},
        },
    }
    external_refs = {}
    for policy in policies:
        if policy.name in {
            "manylinux_2_5_x86_64",
            "manylinux_2_12_x86_64",
            "manylinux_2_17_x86_64",
        }:
            libs = {"libmvec.so.1": Path("/lib64/libmvec-2.28.so")}
        else:
            libs = {}
        blacklist = {}
        external_refs[policy.name] = ExternalReference(libs, blacklist, policy)
    symbol_policies = wheel_abi.get_symbol_policies(
        policies,
        versioned_symbols,
        external_versioned_symbols,
        external_refs,
    )
    max_policy = max(symbol_policy[0] for symbol_policy in symbol_policies)
    assert max_policy.name == "manylinux_2_17_x86_64"


@pytest.mark.parametrize("kind", ["resolved", "unresolved", "warning"])
def test_nonpy_elf_resolution(kind: str, tmp_path: Path, caplog: pytest.LogCaptureFixture) -> None:
    caplog.set_level(logging.WARNING)
    liba_path = tmp_path / "liba.so"
    libb_path = tmp_path / "libb.so"
    liba_path.touch()
    libb_path.touch()
    platform = Platform("", 64, True, "EM_X86_64", Architecture.x86_64, None, None)
    libb = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="libb.so",
        realpath=libb_path,
        platform=platform,
        needed=(),
        rpath=(),
        runpath=(),
        libraries={},
    )
    liba = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="liba.so",
        realpath=liba_path,
        platform=platform,
        needed=("libb.so",),
        rpath=(),
        runpath=(),
        libraries={
            "libb.so": DynamicLibrary("libb.so", None, None),
        },
    )
    libraries = {
        "liba.so": DynamicLibrary("liba.so", liba.path, liba.realpath, liba.platform, liba.needed),
    }
    if kind == "resolved":
        libraries["libb.so"] = DynamicLibrary(
            "libb.so",
            libb.path,
            libb.realpath,
            libb.platform,
            libb.needed,
        )
    elif kind == "unresolved":
        libraries["libb.so"] = DynamicLibrary("libb.so", None, None)
    extension = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="extension.so",
        realpath=Path("extension.so"),
        platform=platform,
        needed=("liba.so",),
        rpath=(),
        runpath=(),
        libraries=libraries,
    )
    pyelf_trees = {extension.realpath: extension}
    nonpy_elftrees = {liba.realpath: liba, libb.realpath: libb}
    wheel_abi._fixup_elf_trees(pyelf_trees, nonpy_elftrees)
    assert pyelf_trees == {extension.realpath: extension}
    if kind == "resolved":
        assert nonpy_elftrees != {liba.realpath: liba, libb.realpath: libb}
        assert nonpy_elftrees[liba.realpath].libraries["libb.so"].realpath == libb.realpath
    else:
        assert nonpy_elftrees == {liba.realpath: liba, libb.realpath: libb}
        assert nonpy_elftrees[liba.realpath].libraries["libb.so"].realpath is None
    warnings = [
        record
        for record in caplog.records
        if "ELF tree" in record.message and record.levelno == logging.WARNING
    ]
    if kind == "warning":
        assert len(warnings) == 1
    else:
        assert len(warnings) == 0


@pytest.mark.parametrize("kind", ["resolved", "unresolved", "warning"])
def test_nonpy_elf_resolution_transitive_needed(
    kind: str,
    tmp_path: Path,
    caplog: pytest.LogCaptureFixture,
) -> None:
    caplog.set_level(logging.WARNING)
    liba_path = tmp_path / "liba.so"
    libb_path = tmp_path / "libb.so"
    libc_path = tmp_path / "libc.so"
    liba_path.touch()
    libb_path.touch()
    libc_path.touch()
    platform = Platform("", 64, True, "EM_X86_64", Architecture.x86_64, None, None)
    libc = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="libc.so",
        realpath=libc_path,
        platform=platform,
        needed=(),
        rpath=(),
        runpath=(),
        libraries={},
    )
    libb = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="libb.so",
        realpath=libb_path,
        platform=platform,
        needed=("libc.so",),
        rpath=(),
        runpath=(),
        libraries={
            "libc.so": DynamicLibrary("libc.so", None, None),
        },
    )
    liba = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="liba.so",
        realpath=liba_path,
        platform=platform,
        needed=("libb.so",),
        rpath=(),
        runpath=(),
        libraries={
            "libb.so": DynamicLibrary("libb.so", None, None),
        },
    )
    libraries = {
        "liba.so": DynamicLibrary("liba.so", liba.path, liba.realpath, liba.platform, liba.needed),
    }
    if kind in {"resolved", "warning"}:
        libraries["libb.so"] = DynamicLibrary(
            "libb.so",
            libb.path,
            libb.realpath,
            libb.platform,
            libb.needed,
        )
        if kind == "resolved":
            libraries["libc.so"] = DynamicLibrary(
                "libc.so",
                libc.path,
                libc.realpath,
                libc.platform,
                libc.needed,
            )
    elif kind == "unresolved":
        libraries["libb.so"] = DynamicLibrary("libb.so", None, None)

    extension = DynamicExecutable(
        interpreter=None,
        libc=None,
        path="extension.so",
        realpath=Path("extension.so"),
        platform=platform,
        needed=("liba.so",),
        rpath=(),
        runpath=(),
        libraries=libraries,
    )
    pyelf_trees = {extension.realpath: extension}
    nonpy_elftrees = {liba.realpath: liba, libb.realpath: libb, libc.realpath: libc}
    wheel_abi._fixup_elf_trees(pyelf_trees, nonpy_elftrees)
    assert pyelf_trees == {extension.realpath: extension}
    if kind in {"resolved", "warning"}:
        assert nonpy_elftrees != {liba.realpath: liba, libb.realpath: libb, libc.realpath: libc}
        assert nonpy_elftrees[liba.realpath].libraries["libb.so"].realpath == libb.realpath
        if kind == "resolved":
            assert nonpy_elftrees[liba.realpath].libraries["libc.so"].realpath == libc.realpath
            assert nonpy_elftrees[libb.realpath].libraries["libc.so"].realpath == libc.realpath
        else:
            assert "libc.so" not in nonpy_elftrees[liba.realpath].libraries
            assert nonpy_elftrees[libb.realpath].libraries["libc.so"].realpath is None
    else:
        assert nonpy_elftrees == {liba.realpath: liba, libb.realpath: libb, libc.realpath: libc}
        assert nonpy_elftrees[liba.realpath].libraries["libb.so"].realpath is None
        assert "libc.so" not in nonpy_elftrees[liba.realpath].libraries
        assert nonpy_elftrees[libb.realpath].libraries["libc.so"].realpath is None

    warnings = [
        record
        for record in caplog.records
        if "ELF tree" in record.message and record.levelno == logging.WARNING
    ]
    if kind == "warning":
        assert len(warnings) == 2
    else:
        assert len(warnings) == 0



from __future__ import annotations

import subprocess
from pathlib import Path

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext


class BuildExt(build_ext):
    def run(self) -> None:
        outputs = self.get_outputs()
        assert len(outputs) == 1
        liba_path = Path(outputs[0]).parent / "liba.so"
        libb_path = Path(outputs[0]).parent / "libb.so"

        cmd = f"gcc -fPIC -shared -o {libb_path} b/b.c"
        subprocess.check_call(cmd.split())
        libb_parent_path = libb_path.parent.resolve(strict=True)
        cmd = f"gcc -fPIC -shared -o {liba_path} -Ib a/a.c -L{libb_parent_path} -lb"
        subprocess.check_call(cmd.split())
        liba_path = liba_path.resolve(strict=True)
        liba_link_path = Path("a/liba.so")
        if liba_link_path.exists():
            liba_link_path.unlink()
        liba_link_path.symlink_to(liba_path)

        super().run()


setup(
    name="testpartialresolution",
    version="0.0.1",
    packages=["testpartialresolution"],
    package_dir={"": "src"},
    cmdclass={"build_ext": BuildExt},
    ext_modules=[
        Extension(
            "testpartialresolution/testpartialresolution",
            sources=["src/testpartialresolution/testpartialresolution.c"],
            extra_link_args=["-Wl,--disable-new-dtags", "-Wl,-rpath=$ORIGIN"],
            include_dirs=["a"],
            libraries=["a"],
            library_dirs=["a"],
        ),
    ],
)



from __future__ import annotations

import importlib
import json
import os
import sys
import zipfile
from argparse import Namespace
from datetime import datetime, timezone
from os.path import isabs
from pathlib import Path
from typing import Any
from unittest.mock import Mock

import pytest

from auditwheel import lddtree, main_repair
from auditwheel.architecture import Architecture
from auditwheel.libc import Libc
from auditwheel.main import main
from auditwheel.wheel_abi import NonPlatformWheelError, analyze_wheel_abi

HERE = Path(__file__).parent.resolve()


@pytest.mark.parametrize(
    ("file", "external_libs", "exclude"),
    [
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libffi.so.5", "libpython2.7.so.1.0"},
            frozenset(),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            set(),
            frozenset(["libffi.so.5", "libpython2.7.so.1.0"]),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libffi.so.5", "libpython2.7.so.1.0"},
            frozenset(["libffi.so.noexist", "libnoexist.so.*"]),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libpython2.7.so.1.0"},
            frozenset(["libffi.so.[4,5]"]),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libffi.so.5", "libpython2.7.so.1.0"},
            frozenset(["libffi.so.[6,7]"]),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libpython2.7.so.1.0"},
            frozenset([f"{HERE}/*"]),
        ),
        (
            "cffi-1.5.0-cp27-none-linux_x86_64.whl",
            {"libpython2.7.so.1.0"},
            frozenset(["libffi.so.*"]),
        ),
        ("cffi-1.5.0-cp27-none-linux_x86_64.whl", set(), frozenset(["*"])),
        (
            "python_snappy-0.5.2-pp260-pypy_41-linux_x86_64.whl",
            {"libsnappy.so.1"},
            frozenset(),
        ),
    ],
)
def test_analyze_wheel_abi(file, external_libs, exclude):
    # If exclude libs contain path, LD_LIBRARY_PATH need to be modified to find the libs
    # `lddtree.load_ld_paths` needs to be reloaded for it's `lru_cache`-ed.
    modify_ld_library_path = any(isabs(e) for e in exclude)

    with pytest.MonkeyPatch.context() as cp:
        if modify_ld_library_path:
            cp.setenv("LD_LIBRARY_PATH", f"{HERE}")
            importlib.reload(lddtree)

        winfo = analyze_wheel_abi(
            Libc.GLIBC,
            Architecture.x86_64,
            HERE / file,
            exclude,
            disable_isa_ext_check=False,
            allow_graft=True,
        )
        assert set(winfo.external_refs["manylinux_2_5_x86_64"].libs) == external_libs, (
            f"{HERE}, {exclude}, {os.environ}"
        )

    if modify_ld_library_path:
        importlib.reload(lddtree)


def test_analyze_wheel_abi_pyfpe():
    winfo = analyze_wheel_abi(
        Libc.GLIBC,
        Architecture.x86_64,
        HERE / "fpewheel-0.0.0-cp35-cp35m-linux_x86_64.whl",
        frozenset(),
        disable_isa_ext_check=False,
        allow_graft=True,
    )
    # for external symbols, it could get manylinux1
    assert winfo.sym_policy.name == "manylinux_2_5_x86_64"
    # but for having the pyfpe reference, it gets just linux
    assert winfo.pyfpe_policy.name == "linux_x86_64"
    assert winfo.overall_policy.name == "linux_x86_64"


def test_show_wheel_abi_pyfpe(monkeypatch, capsys):
    wheel = str(HERE / "fpewheel-0.0.0-cp35-cp35m-linux_x86_64.whl")
    monkeypatch.setattr(sys, "platform", "linux")
    monkeypatch.setattr(Architecture, "detect", lambda: Architecture.x86_64)
    monkeypatch.setattr(sys, "argv", ["auditwheel", "show", wheel])
    assert main() == 0
    captured = capsys.readouterr()
    assert "This wheel uses the PyFPE_jbuf function" in captured.out


def test_analyze_wheel_abi_bad_architecture():
    with pytest.raises(NonPlatformWheelError):
        analyze_wheel_abi(
            Libc.GLIBC,
            Architecture.aarch64,
            HERE / "fpewheel-0.0.0-cp35-cp35m-linux_x86_64.whl",
            frozenset(),
            disable_isa_ext_check=False,
            allow_graft=True,
        )


def test_analyze_wheel_abi_static_exe(caplog):
    result = analyze_wheel_abi(
        None,
        None,
        HERE
        / "patchelf-0.17.2.1-py2.py3-none-manylinux_2_5_x86_64.manylinux1_x86_64.musllinux_1_1_x86_64.whl",  # noqa: E501
        frozenset(),
        disable_isa_ext_check=False,
        allow_graft=False,
    )
    assert "setting architecture to x86_64" in caplog.text
    assert "couldn't detect wheel libc, defaulting to" in caplog.text
    assert result.policies.architecture == Architecture.x86_64
    if Libc.detect() == Libc.MUSL:
        assert result.policies.libc == Libc.MUSL
        assert result.overall_policy.name.startswith("musllinux_1_")
    else:
        assert result.policies.libc == Libc.GLIBC
        assert result.overall_policy.name == "manylinux_2_5_x86_64"


@pytest.mark.parametrize(
    "timestamp",
    [
        (0, 315532800),  # zip timestamp starts 1980-01-01, not 1970-01-01
        (315532799, 315532800),  # zip timestamp starts 1980-01-01, not 1970-01-01
        (315532801, 315532800),  # zip timestamp round odd seconds down to even seconds
        (315532802, 315532802),
        (650203201, 650203200),  # zip timestamp round odd seconds down to even seconds
    ],
)
def test_wheel_source_date_epoch(timestamp, tmp_path, monkeypatch):
    wheel_path = HERE / "arch-wheels/musllinux_1_2/testsimple-0.0.1-cp312-cp312-linux_x86_64.whl"
    wheel_output_path = tmp_path / "out"
    args = Namespace(
        LIB_SDIR=".libs",
        ONLY_PLAT=False,
        PLAT="auto",
        STRIP=False,
        UPDATE_TAGS=True,
        WHEEL_DIR=wheel_output_path,
        WHEEL_FILE=[wheel_path],
        EXCLUDE=[],
        LDPATHS=None,
        DISABLE_ISA_EXT_CHECK=False,
        ZIP_COMPRESSION_LEVEL=6,
        cmd="repair",
        func=Mock(),
        prog="auditwheel",
        verbose=1,
    )

    monkeypatch.setenv("SOURCE_DATE_EPOCH", str(timestamp[0]))
    main_repair.execute(args, Mock())
    output_wheel, *_ = list(wheel_output_path.glob("*.whl"))
    with zipfile.ZipFile(output_wheel) as wheel_file:
        for file in wheel_file.infolist():
            file_date_time = datetime(*file.date_time, tzinfo=timezone.utc)
            assert file_date_time.timestamp() == timestamp[1]


def test_libpython(tmp_path, caplog):
    wheel = HERE / "python_mscl-67.0.1.0-cp313-cp313-manylinux2014_aarch64.whl"
    args = Namespace(
        LIB_SDIR=".libs",
        ONLY_PLAT=False,
        PLAT="auto",
        STRIP=False,
        UPDATE_TAGS=True,
        WHEEL_DIR=tmp_path,
        WHEEL_FILE=[wheel],
        EXCLUDE=[],
        LDPATHS=None,
        DISABLE_ISA_EXT_CHECK=False,
        ZIP_COMPRESSION_LEVEL=6,
        cmd="repair",
        func=Mock(),
        prog="auditwheel",
        verbose=0,
    )
    main_repair.execute(args, Mock())
    assert "Removing libpython3.13.so.1.0 dependency from python_mscl/_mscl.so" in caplog.text
    assert tuple(path.name for path in tmp_path.glob("*.whl")) == (
        "python_mscl-67.0.1.0-cp313-cp313-manylinux2014_aarch64.manylinux_2_31_aarch64.whl",
    )


def test_main_lddtree(
    monkeypatch: pytest.MonkeyPatch,
    tmp_path: Path,
    caplog: pytest.LogCaptureFixture,
) -> None:
    wheel_path = (
        HERE
        / "patchelf-0.17.2.1-py2.py3-none-manylinux_2_5_x86_64.manylinux1_x86_64.musllinux_1_1_x86_64.whl"  # noqa: E501
    )
    patchelf_path = tmp_path / "patchelf-0.17.2.1.data/scripts/patchelf"
    with zipfile.ZipFile(wheel_path) as f:
        f.extract(str(patchelf_path.relative_to(tmp_path)), tmp_path)
    patchelf_path = patchelf_path.resolve(strict=True)

    monkeypatch.setattr(sys, "platform", "linux")
    monkeypatch.setattr(Architecture, "detect", lambda: Architecture.x86_64)
    monkeypatch.setattr(sys, "argv", ["auditwheel", "lddtree", str(patchelf_path)])
    assert main() == 0
    assert len(caplog.messages) == 1
    actual_json = json.loads(caplog.messages[0])
    expected_json: Any = {
        "interpreter": None,
        "libc": None,
        "path": str(patchelf_path),
        "realpath": str(patchelf_path),
        "platform": {
            "_elf_osabi": "ELFOSABI_SYSV",
            "_elf_class": 64,
            "_elf_little_endian": True,
            "_elf_machine": "EM_X86_64",
            "_base_arch": "<Architecture.x86_64: 'x86_64'>",
            "_ext_arch": None,
            "_error_msg": None,
        },
        "needed": [],
        "rpath": [],
        "runpath": [],
        "libraries": {},
    }
    assert expected_json == actual_json


def test_weak_symbols_not_blacklisted() -> None:
    # https://github.com/pypa/auditwheel/issues/663
    # the cryptography wheel overall policy was misclassified as manylinux_2_24_x86_64
    # in auditwheel 6.5.1 because it uses the undefined weak symbol '__cxa_thread_atexit_impl'
    result = analyze_wheel_abi(
        None,
        None,
        HERE / "cryptography-46.0.3-cp38-abi3-manylinux2014_x86_64.manylinux_2_17_x86_64.whl",
        frozenset(),
        disable_isa_ext_check=False,
        allow_graft=False,
    )
    assert result.policies.libc == Libc.GLIBC
    assert result.policies.architecture == Architecture.x86_64
    assert result.overall_policy.name == "manylinux_2_17_x86_64"



from dataclasses import dataclass
from enum import Enum
from json import loads

import pytest

from auditwheel.json import dumps


def test_dataclass():
    @dataclass(frozen=True)
    class Dummy:
        first: str = "val0"
        second: int = 2

    assert loads(dumps(Dummy())) == {"first": "val0", "second": 2}


def test_enum():
    class Dummy(Enum):
        value: str

        TEST = "dummy"

        def __repr__(self):
            return self.value

    assert Dummy.TEST.value == loads(dumps(Dummy.TEST))


def test_frozenset():
    obj = frozenset((3, 9, 6, 5, 21))
    data = loads(dumps(obj))
    assert data == sorted(obj)


def test_invalid_type():
    class Dummy:
        pass

    with pytest.raises(TypeError):
        dumps(Dummy())



from __future__ import annotations

import re
from pathlib import Path
from unittest.mock import patch

import pytest

from auditwheel.condatools import InCondaPkg, InCondaPkgCtx


@patch("auditwheel.condatools.tarbz2todir")
def test_in_condapkg(_):  # noqa: PT019
    with InCondaPkg(Path("/fakepath")):
        assert True


@patch("auditwheel.condatools.tarbz2todir")
def test_in_condapkg_context(_):  # noqa: PT019
    with InCondaPkgCtx(Path("/fakepath")) as conda_pkg:
        # mock info/files
        files_path = conda_pkg.path / "info" / "files"
        files_path.parent.mkdir()
        files_path.write_text("file1\nfile2\n\n")
        # This returns empty lines so we have count with those as well. This
        # might be a subtle bug in the implementation.
        files = conda_pkg.iter_files()
        assert len(files) == 3
        assert "file1" in files
        assert "file2" in files


@patch("auditwheel.condatools.tarbz2todir")
def test_in_condapkg_context_no_manager(_):  # noqa: PT019
    conda_pkg = InCondaPkgCtx(Path("/fakepath"))
    with pytest.raises(
        ValueError,
        match=re.escape("This function should be called from context manager"),
    ):
        conda_pkg.iter_files()



from __future__ import annotations

import argparse
import logging
import os
import subprocess
import time
import zipfile
from datetime import datetime, timezone
from pathlib import Path
from typing import TYPE_CHECKING, Any, TypeVar

if TYPE_CHECKING:
    from collections.abc import Generator, Iterable

_T = TypeVar("_T")

logger = logging.getLogger(__name__)


def unique_by_index(sequence: Iterable[_T]) -> list[_T]:
    """Unique elements in `sequence` in the order in which they occur

    Parameters
    ----------
    sequence : iterable

    Returns
    -------
    uniques : list
        unique elements of sequence, ordered by the order in which the element
        occurs in `sequence`

    """
    uniques = []
    for element in sequence:
        if element not in uniques:
            uniques.append(element)
    return uniques


def walk(topdir: Path) -> Generator[tuple[Path, list[str], list[str]]]:
    """Wrapper for `os.walk` with outputs in reproducible order

    Parameters
    ----------
    topdir : Path
        Root of the directory tree

    Yields
    ------
    dirpath : Path
        Path to a directory
    dirnames : list[str]
        List of subdirectory names in `dirpath`
    filenames : list[str]
        List of non-directory file names in `dirpath`

    """
    topdir = topdir.resolve(strict=True)
    for dirpath_, dirnames, filenames in os.walk(topdir):
        dirpath = Path(dirpath_)
        # sort list of dirnames in-place such that `os.walk`
        # will recurse into subdirectories in reproducible order
        dirnames.sort()
        # recurse into any top-level .dist-info subdirectory last
        if dirpath == topdir:
            subdirs = []
            dist_info = []
            for dir_ in dirnames:
                if dir_.endswith(".dist-info"):
                    dist_info.append(dir_)
                else:
                    subdirs.append(dir_)
            dirnames[:] = subdirs
            dirnames.extend(dist_info)
            del dist_info
        # sort list of filenames for iteration in reproducible order
        filenames.sort()
        # list any dist-info/RECORD file last
        if (
            dirpath.name.endswith(".dist-info")
            and dirpath.parent == topdir
            and "RECORD" in filenames
        ):
            filenames.remove("RECORD")
            filenames.append("RECORD")
        yield dirpath, dirnames, filenames


def zip2dir(zip_fname: Path, out_dir: Path) -> None:
    """Extract `zip_fname` into output directory `out_dir`

    Parameters
    ----------
    zip_fname : str
        Filename of zip archive to write
    out_dir : str
        Directory path containing files to go in the zip archive

    """
    start = time.perf_counter()
    with zipfile.ZipFile(zip_fname, "r") as z:
        for name in z.namelist():
            member = z.getinfo(name)
            extracted_path = Path(z.extract(member, out_dir))
            attr = member.external_attr >> 16
            if member.is_dir():
                # this is always rebuilt as 755 by dir2zip
                extracted_path.chmod(0o755)
            elif attr != 0:
                attr &= 511  # only keep permission bits
                attr |= 6 << 6  # at least read/write for current user
                extracted_path.chmod(attr)
    logger.debug(
        "zip2dir from %s to %s takes %s",
        zip_fname,
        out_dir,
        time.perf_counter() - start,
    )


def dir2zip(
    in_dir: Path,
    zip_fname: Path,
    zip_compression_level: int,
    date_time: datetime | None,
) -> None:
    """Make a zip file `zip_fname` with contents of directory `in_dir`

    The recorded filenames are relative to `in_dir`, so doing a standard zip
    unpack of the resulting `zip_fname` in an empty directory will result in
    the original directory contents.

    Parameters
    ----------
    in_dir : Path
        Directory path containing files to go in the zip archive
    zip_fname : Path
        Filename of zip archive to write
    zip_compression_level: int
        zlib.Z_DEFAULT_COMPRESSION (-1 aka. level 6) balances speed and size.
        zlib.Z_NO_COMPRESSION (O) for some test builds that needs no compression at all
        zlib.Z_BEST_COMPRESSION (9) for bandwidth-constrained or large amount of downloads
    date_time : Optional[datetime]
        Time stamp to set on each file in the archive

    """
    start = time.perf_counter()
    in_dir = in_dir.resolve(strict=True)
    if date_time is None:
        st = in_dir.stat()
        date_time = datetime.fromtimestamp(st.st_mtime, tz=timezone.utc)
    date_time_args = date_time.timetuple()[:6]
    if date_time_args < (1980, 1, 1, 0, 0, 0):
        logger.warning("dir2zip, clipping timestamp to 1980-01-01")
        date_time_args = (1980, 1, 1, 0, 0, 0)
    compression = zipfile.ZIP_DEFLATED
    with zipfile.ZipFile(zip_fname, "w", compression=compression) as z:
        for dname, _, files in walk(in_dir):
            if dname != in_dir:
                out_dname = f"{dname.relative_to(in_dir)}/"
                zinfo = zipfile.ZipInfo.from_file(dname, out_dname)
                zinfo.date_time = date_time_args
                z.writestr(zinfo, b"")
            for file in files:
                fname = dname / file
                out_fname = fname.relative_to(in_dir)
                zinfo = zipfile.ZipInfo.from_file(fname, out_fname)
                zinfo.date_time = date_time_args
                zinfo.compress_type = compression
                with fname.open("rb") as fp:
                    z.writestr(zinfo, fp.read(), compresslevel=zip_compression_level)
    logger.debug(
        "dir2zip from %s to %s takes %s",
        in_dir,
        zip_fname,
        time.perf_counter() - start,
    )


def tarbz2todir(tarbz2_fname: Path, out_dir: Path) -> None:
    """Extract `tarbz2_fname` into output directory `out_dir`"""
    subprocess.check_output(["tar", "xjf", tarbz2_fname, "-C", out_dir])


class EnvironmentDefault(argparse.Action):
    """Get values from environment variable."""

    def __init__(
        self,
        *,
        env: str,
        required: bool = True,
        default: str | None = None,
        choices: Iterable[str] | None = None,
        type: type | None = None,  # noqa: A002
        **kwargs: Any,  # noqa: ANN401
    ) -> None:
        self.env_default = os.environ.get(env)
        self.env = env
        if self.env_default:
            if type:
                try:
                    self.env_default = type(self.env_default)
                except Exception:  # noqa: BLE001
                    self.option_strings = kwargs["option_strings"]
                    args = {
                        "value": self.env_default,
                        "type": type,
                        "env": self.env,
                    }
                    msg = (
                        "invalid type: %(value)r from environment variable "
                        "%(env)r cannot be converted to %(type)r"
                    )
                    raise argparse.ArgumentError(self, msg % args) from None
            default = self.env_default
        if self.env_default is not None and choices is not None and self.env_default not in choices:
            self.option_strings = kwargs["option_strings"]
            args = {
                "value": self.env_default,
                "choices": ", ".join(map(repr, choices)),
                "env": self.env,
            }
            msg = (
                "invalid choice: %(value)r from environment variable "
                "%(env)r (choose from %(choices)s)"
            )
            raise argparse.ArgumentError(self, msg % args)

        if default is not None:
            required = False

        super().__init__(
            default=default,
            required=required,
            choices=choices,
            type=type,
            **kwargs,
        )

    def __call__(
        self,
        parser: argparse.ArgumentParser,  # noqa: ARG002
        namespace: argparse.Namespace,
        values: Any,  # noqa: ANN401
        option_string: str | None = None,  # noqa: ARG002
    ) -> None:
        setattr(namespace, self.dest, values)


def is_subdir(path: str | Path | None, root: str | Path) -> bool:
    if path is None:
        return False

    path = Path(path).resolve()
    root = Path(root).resolve()

    return root == path or root in path.parents



from __future__ import annotations

import argparse
import lzma
import zipfile
import zlib
from pathlib import Path

import pytest

from auditwheel.tools import EnvironmentDefault, dir2zip, is_subdir, zip2dir


@pytest.mark.parametrize(
    ("environ", "passed", "expected"),
    [
        (None, None, "manylinux1"),
        (None, "manylinux2010", "manylinux2010"),
        ("manylinux2010", None, "manylinux2010"),
        ("manylinux2010", "linux", "linux"),
    ],
)
def test_plat_environment_action(
    monkeypatch: pytest.MonkeyPatch,
    environ: str | None,
    passed: str | None,
    expected: str,
) -> None:
    choices = ["linux", "manylinux1", "manylinux2010"]
    argv = []
    if passed:
        argv = ["--plat", passed]
    if environ:
        monkeypatch.setenv("AUDITWHEEL_PLAT", environ)
    p = argparse.ArgumentParser()
    p.add_argument(
        "--plat",
        action=EnvironmentDefault,
        env="AUDITWHEEL_PLAT",
        dest="PLAT",
        choices=choices,
        default="manylinux1",
    )
    args = p.parse_args(argv)
    assert expected == args.PLAT


_all_zip_level: list[int] = list(
    range(zlib.Z_NO_COMPRESSION, zlib.Z_BEST_COMPRESSION + 1),
)


@pytest.mark.parametrize(
    ("environ", "passed", "expected"),
    [
        (None, None, -1),
        (0, None, 0),
        (0, 1, 1),
        (6, 1, 1),
    ],
)
def test_zip_environment_action(
    monkeypatch: pytest.MonkeyPatch,
    environ: int | None,
    passed: int | None,
    expected: int,
) -> None:
    choices = _all_zip_level
    argv = []
    if passed is not None:
        argv = ["--zip-compression-level", str(passed)]
    if environ is not None:
        monkeypatch.setenv("AUDITWHEEL_ZIP_COMPRESSION_LEVEL", str(environ))
    p = argparse.ArgumentParser()
    p.add_argument(
        "-z",
        "--zip-compression-level",
        action=EnvironmentDefault,
        metavar="zip",
        env="AUDITWHEEL_ZIP_COMPRESSION_LEVEL",
        dest="zip",
        type=int,
        help="Compress level to be used to create zip file.",
        choices=choices,
        default=zlib.Z_DEFAULT_COMPRESSION,
    )
    args = p.parse_args(argv)
    assert expected == args.zip


def test_environment_action_invalid_plat_env(monkeypatch: pytest.MonkeyPatch) -> None:
    choices = ["linux", "manylinux1", "manylinux2010"]
    monkeypatch.setenv("AUDITWHEEL_PLAT", "foo")
    p = argparse.ArgumentParser()
    with pytest.raises(argparse.ArgumentError):
        p.add_argument(
            "--plat",
            action=EnvironmentDefault,
            env="AUDITWHEEL_PLAT",
            dest="PLAT",
            choices=choices,
            default="manylinux1",
        )


def test_environment_action_invalid_zip_env(monkeypatch: pytest.MonkeyPatch) -> None:
    choices = _all_zip_level
    monkeypatch.setenv("AUDITWHEEL_ZIP_COMPRESSION_LEVEL", "foo")
    p = argparse.ArgumentParser()
    with pytest.raises(argparse.ArgumentError):
        p.add_argument(
            "-z",
            "--zip-compression-level",
            action=EnvironmentDefault,
            metavar="zip",
            env="AUDITWHEEL_ZIP_COMPRESSION_LEVEL",
            dest="zip",
            type=int,
            help="Compress level to be used to create zip file.",
            choices=choices,
            default=zlib.Z_DEFAULT_COMPRESSION,
        )
    monkeypatch.setenv("AUDITWHEEL_ZIP_COMPRESSION_LEVEL", "10")
    with pytest.raises(argparse.ArgumentError):
        p.add_argument(
            "-z",
            "--zip-compression-level",
            action=EnvironmentDefault,
            metavar="zip",
            env="AUDITWHEEL_ZIP_COMPRESSION_LEVEL",
            dest="zip",
            type=int,
            help="Compress level to be used to create zip file.",
            choices=choices,
            default=zlib.Z_DEFAULT_COMPRESSION,
        )


def _write_test_permissions_zip(path: Path) -> None:
    source_zip_xz = Path(__file__).parent / "test-permissions.zip.xz"
    with lzma.open(source_zip_xz) as f:
        path.write_bytes(f.read())


def _check_permissions(path: Path) -> None:
    for i in range(8):
        for j in range(8):
            for k in range(8):
                mode = (path / f"{i}{j}{k}.f").stat().st_mode
                assert ((mode >> 6) & 7) == (i | 6)  # always read/write
                assert ((mode >> 3) & 7) == j
                assert ((mode >> 0) & 7) == k
                mode = (path / f"{i}{j}{k}.d").stat().st_mode
                assert ((mode >> 6) & 7) == 7  # always read/write/execute
                assert ((mode >> 3) & 7) == 5  # always read/execute
                assert ((mode >> 0) & 7) == 5  # always read/execute


def test_zip2dir_permissions(tmp_path: Path) -> None:
    source_zip = tmp_path / "test-permissions.zip"
    _write_test_permissions_zip(source_zip)
    extract_path = tmp_path / "unzip"
    zip2dir(source_zip, extract_path)
    _check_permissions(extract_path)


def test_zip2dir_round_trip_permissions(tmp_path: Path) -> None:
    source_zip = tmp_path / "test-permissions.zip"
    _write_test_permissions_zip(source_zip)
    extract_path = tmp_path / "unzip2"
    zip2dir(source_zip, tmp_path / "unzip1")
    dir2zip(tmp_path / "unzip1", tmp_path / "tmp.zip", zlib.Z_DEFAULT_COMPRESSION, None)
    zip2dir(tmp_path / "tmp.zip", extract_path)
    _check_permissions(extract_path)


def test_dir2zip_deflate(tmp_path: Path) -> None:
    buffer = b"\0" * 1024 * 1024
    input_dir = tmp_path / "input_dir"
    input_dir.mkdir()
    input_file = input_dir / "zeros.bin"
    input_file.write_bytes(buffer)
    output_file = tmp_path / "ouput.zip"
    dir2zip(input_dir, output_file, zlib.Z_DEFAULT_COMPRESSION, None)
    assert output_file.stat().st_size < len(buffer) / 4


def test_dir2zip_folders(tmp_path: Path) -> None:
    input_dir = tmp_path / "input_dir"
    input_dir.mkdir()
    dist_info_folder = input_dir / "dummy-1.0.dist-info"
    dist_info_folder.mkdir()
    dist_info_folder.joinpath("METADATA").write_text("")
    empty_folder = input_dir / "dummy" / "empty"
    empty_folder.mkdir(parents=True)
    output_file = tmp_path / "output.zip"
    dir2zip(input_dir, output_file, zlib.Z_DEFAULT_COMPRESSION, None)
    expected_dirs = {"dummy/", "dummy/empty/", "dummy-1.0.dist-info/"}
    with zipfile.ZipFile(output_file, "r") as z:
        assert len(z.filelist) == 4
        for info in z.filelist:
            if info.is_dir():
                assert info.filename in expected_dirs
                expected_dirs.remove(info.filename)
            else:
                assert info.filename == "dummy-1.0.dist-info/METADATA"
    assert len(expected_dirs) == 0


def test_is_subdir(tmp_path: Path) -> None:
    root = tmp_path / "root"
    subdir = root / "subdir"
    subdir.mkdir(parents=True)
    assert is_subdir(subdir, root)
    assert is_subdir(root, root)
    assert not is_subdir(None, root)
    assert not is_subdir(tmp_path, root)



from __future__ import annotations

import re
from itertools import chain
from shutil import which
from subprocess import CalledProcessError, check_call, check_output
from typing import TYPE_CHECKING

from auditwheel.wheeltools import android_api_level

if TYPE_CHECKING:
    from pathlib import Path


class ElfPatcher:
    def __init__(self, platform: str) -> None:
        self.platform = platform

    def replace_needed(self, file_name: Path, *old_new_pairs: tuple[str, str]) -> None:
        raise NotImplementedError

    def remove_needed(self, file_name: Path, *sonames: str) -> None:
        raise NotImplementedError

    def set_soname(self, file_name: Path, new_so_name: str) -> None:
        raise NotImplementedError

    def check_rpath_support(self) -> None:
        # https://android.googlesource.com/platform/bionic/+/refs/heads/main/android-changes-for-ndk-developers.md
        if self.platform.startswith("android") and android_api_level(self.platform) < 24:
            msg = "Grafting libraries with RUNPATH requires API level 24 or higher"
            raise ValueError(msg)

    def set_rpath(self, file_name: Path, rpath: str) -> None:
        raise NotImplementedError

    def get_rpath(self, file_name: Path) -> str:
        raise NotImplementedError

    def clear_rpath(self, file_name: Path) -> None:
        raise NotImplementedError


def _verify_patchelf() -> None:
    """This function looks for the ``patchelf`` external binary in the PATH,
    checks for the required version, and throws an exception if a proper
    version can't be found. Otherwise, silence is golden
    """
    if not which("patchelf"):
        msg = "Cannot find required utility `patchelf` in PATH"
        raise ValueError(msg)
    try:
        version = check_output(["patchelf", "--version"]).decode("utf-8")
    except CalledProcessError:
        msg = "Could not call `patchelf` binary"
        raise ValueError(msg) from None

    m = re.match(r"patchelf\s+(\d+(.\d+)?)", version)
    if m and tuple(int(x) for x in m.group(1).split(".")) >= (0, 14):
        return
    msg = f"patchelf {version} found. auditwheel repair requires patchelf >= 0.14."
    raise ValueError(msg)


class Patchelf(ElfPatcher):
    def __init__(self, platform: str = "") -> None:
        super().__init__(platform)
        _verify_patchelf()

    def replace_needed(self, file_name: Path, *old_new_pairs: tuple[str, str]) -> None:
        check_call(
            [
                "patchelf",
                *chain.from_iterable(("--replace-needed", *pair) for pair in old_new_pairs),
                file_name,
            ],
        )

    def remove_needed(self, file_name: Path, *sonames: str) -> None:
        check_call(
            [
                "patchelf",
                *chain.from_iterable(("--remove-needed", soname) for soname in sonames),
                file_name,
            ],
        )

    def set_soname(self, file_name: Path, new_so_name: str) -> None:
        check_call(["patchelf", "--set-soname", new_so_name, file_name])

    def set_rpath(self, file_name: Path, rpath: str) -> None:
        self.check_rpath_support()

        set_args: list[str | Path] = ["patchelf", "--force-rpath", "--set-rpath", rpath, file_name]
        if self.platform.startswith("android"):
            # Android supports only RUNPATH, not RPATH.
            set_args.remove("--force-rpath")

        # we only want an RPATH, remove RUNPATH/RPATH altogether in a 1st pass
        check_call(["patchelf", "--remove-rpath", file_name])
        check_call(set_args)

    def get_rpath(self, file_name: Path) -> str:
        return check_output(["patchelf", "--print-rpath", file_name]).decode("utf-8").strip()

    def clear_rpath(self, file_name: Path) -> None:
        check_call(["patchelf", "--remove-rpath", file_name])



from __future__ import annotations

import os
import subprocess

from setuptools import Extension, setup
from setuptools.command.build_ext import build_ext

USE_DTAGS = os.getenv("DTAG") != "none"


class BuildExt(build_ext):
    def run(self) -> None:
        use_runpath = os.getenv("DTAG") == "runpath"
        dtags_kind_flag = "--enable-new-dtags" if use_runpath else "--disable-new-dtags"
        dtags = f"-Wl,{dtags_kind_flag} -Wl,-rpath=$ORIGIN/../b" if USE_DTAGS else ""

        cmd = f"gcc -fPIC -shared -o b/libb.so {dtags} b/b.c"
        subprocess.check_call(cmd.split())

        dtags = f"-Wl,{dtags_kind_flag} -Wl,-rpath=$ORIGIN/../b" if USE_DTAGS else ""
        cmd = f"gcc -fPIC -shared -o a/liba.so {dtags} -Ib a/a.c -Lb -lb"
        subprocess.check_call(cmd.split())
        super().run()


setup(
    name="testrpath",
    version="0.0.1",
    packages=["testrpath"],
    package_dir={"": "src"},
    cmdclass={"build_ext": BuildExt},
    ext_modules=[
        Extension(
            "testrpath/testrpath",
            sources=["src/testrpath/testrpath.c"],
            include_dirs=["a"],
            libraries=["a"],
            library_dirs=["a"] if USE_DTAGS else ["a", "b"],
        ),
    ],
)



import platform
import struct
import sys

import pytest

from auditwheel.architecture import Architecture


@pytest.mark.parametrize(
    ("sys_platform", "reported_arch", "expected_arch"),
    [
        ("linux", "armv7l", Architecture.armv7l),
        ("linux", "armv8l", Architecture.armv7l),
        ("linux", "aarch64", Architecture.armv7l),
        ("linux", "i686", Architecture.i686),
        ("linux", "x86_64", Architecture.i686),
        ("win32", "x86", Architecture.i686),
        ("win32", "AMD64", Architecture.i686),
    ],
)
def test_32bits_arch_name(sys_platform, reported_arch, expected_arch, monkeypatch):
    monkeypatch.setattr(sys, "platform", sys_platform)
    monkeypatch.setattr(platform, "machine", lambda: reported_arch)
    machine = Architecture.detect(bits=32)
    assert machine == expected_arch


@pytest.mark.parametrize(
    ("sys_platform", "reported_arch", "expected_arch"),
    [
        ("linux", "armv8l", Architecture.aarch64),
        ("linux", "aarch64", Architecture.aarch64),
        ("linux", "ppc64le", Architecture.ppc64le),
        ("linux", "i686", Architecture.x86_64),
        ("linux", "x86_64", Architecture.x86_64),
        ("darwin", "arm64", Architecture.aarch64),
        ("darwin", "x86_64", Architecture.x86_64),
        ("win32", "ARM64", Architecture.aarch64),
        ("win32", "AMD64", Architecture.x86_64),
    ],
)
def test_64bits_arch_name(sys_platform, reported_arch, expected_arch, monkeypatch):
    monkeypatch.setattr(sys, "platform", sys_platform)
    monkeypatch.setattr(platform, "machine", lambda: reported_arch)
    machine = Architecture.detect(bits=64)
    assert machine == expected_arch


@pytest.mark.parametrize(
    ("maxsize", "sizeof_voidp", "expected"),
    [
        # 64-bit
        (9223372036854775807, 8, Architecture.x86_64),
        # 32-bit
        (2147483647, 4, Architecture.i686),
        # 64-bit w/ 32-bit sys.maxsize: GraalPy, IronPython, Jython
        (2147483647, 8, Architecture.x86_64),
    ],
)
def test_arch_name_bits(maxsize, sizeof_voidp, expected, monkeypatch):
    def _calcsize(fmt):
        assert fmt == "P"
        return sizeof_voidp

    monkeypatch.setattr(platform, "machine", lambda: "x86_64")
    monkeypatch.setattr(sys, "maxsize", maxsize)
    monkeypatch.setattr(struct, "calcsize", _calcsize)
    machine = Architecture.detect()
    assert machine == expected


@pytest.mark.parametrize(
    ("smaller", "larger"),
    [
        (Architecture.x86_64, Architecture.x86_64_v4),
        (Architecture.x86_64, Architecture.x86_64),
        (Architecture.x86_64, Architecture.x86_64_v2),
        (Architecture.x86_64_v2, Architecture.x86_64_v3),
        (Architecture.x86_64_v3, Architecture.x86_64_v4),
    ],
)
def test_order_valid(smaller, larger):
    assert smaller.is_subset(larger)
    assert larger.is_superset(smaller)


@pytest.mark.parametrize(
    ("smaller", "larger"),
    [
        (Architecture.x86_64, Architecture.x86_64_v4),
        (Architecture.x86_64, Architecture.x86_64_v2),
        (Architecture.x86_64_v2, Architecture.x86_64_v3),
        (Architecture.x86_64_v3, Architecture.x86_64_v4),
        (Architecture.aarch64, Architecture.x86_64),
        (Architecture.x86_64, Architecture.aarch64),
    ],
)
def test_order_invalid(smaller, larger):
    assert not smaller.is_superset(larger)
    assert not larger.is_subset(smaller)



from __future__ import annotations

import subprocess
import sys
from importlib import metadata

import pytest

from auditwheel.architecture import Architecture
from auditwheel.libc import Libc, LibcVersion
from auditwheel.main import main


def test_help(monkeypatch, capsys):
    # GIVEN
    monkeypatch.setattr(sys, "argv", ["auditwheel"])
    # handle running tests using 'python -m pytest' rather than just 'pytest' on Python 3.14+
    monkeypatch.delattr(sys.modules.get("__main__"), "__spec__", raising=False)

    # WHEN
    retval = main()

    # THEN
    assert retval is None
    captured = capsys.readouterr()
    assert "usage: auditwheel [-h] [-V] [-v] command ..." in captured.out


@pytest.mark.parametrize("function", ["show", "repair"])
def test_unexisting_wheel(monkeypatch, capsys, tmp_path, function):
    monkeypatch.setattr(sys, "platform", "linux")
    monkeypatch.setattr(Architecture, "detect", lambda: Architecture.x86_64)
    wheel = str(tmp_path / "not-a-file.whl")
    monkeypatch.setattr(sys, "argv", ["auditwheel", function, wheel])

    with pytest.raises(SystemExit):
        main()

    captured = capsys.readouterr()
    assert "No such file" in captured.err


@pytest.mark.parametrize(
    ("libc", "filename", "plat", "message"),
    [
        (
            Libc.GLIBC,
            "foo-1.0-py3-none-manylinux1_aarch64.whl",
            "manylinux_2_28_x86_64",
            "can't repair wheel foo-1.0-py3-none-manylinux1_aarch64.whl with aarch64 architecture to a wheel targeting x86_64",  # noqa: E501
        ),
        (
            Libc.GLIBC,
            "foo-1.0-py3-none-musllinux_1_1_x86_64.whl",
            "manylinux_2_28_x86_64",
            "can't repair wheel foo-1.0-py3-none-musllinux_1_1_x86_64.whl with MUSL libc to a wheel targeting GLIBC",  # noqa: E501
        ),
        (
            Libc.MUSL,
            "foo-1.0-py3-none-manylinux1_x86_64.whl",
            "musllinux_1_1_x86_64",
            "can't repair wheel foo-1.0-py3-none-manylinux1_x86_64.whl with GLIBC libc to a wheel targeting MUSL",  # noqa: E501
        ),
    ],
)
def test_repair_wheel_mismatch(
    monkeypatch,
    capsys,
    tmp_path,
    libc,
    filename,
    plat,
    message,
):
    monkeypatch.setattr(sys, "platform", "linux")
    monkeypatch.setattr(Architecture, "detect", lambda: Architecture.x86_64)
    monkeypatch.setattr(Libc, "detect", lambda: libc)
    monkeypatch.setattr(Libc, "get_current_version", lambda _: LibcVersion(1, 1))
    wheel = tmp_path / filename
    wheel.write_text("")
    monkeypatch.setattr(
        sys,
        "argv",
        ["auditwheel", "repair", "--plat", plat, str(wheel)],
    )

    with pytest.raises(SystemExit):
        main()

    captured = capsys.readouterr()
    assert message in captured.err


def test_main_module() -> None:
    version = metadata.version("auditwheel")
    result = subprocess.run(
        [sys.executable, "-m", "auditwheel", "-V"],
        check=True,
        capture_output=True,
        text=True,
    )
    assert result.stdout.startswith(f"auditwheel {version}")



from __future__ import annotations

import logging
import os
import re
import subprocess
from dataclasses import dataclass
from enum import Enum
from pathlib import Path

from auditwheel.error import InvalidLibcError

logger = logging.getLogger(__name__)


@dataclass(frozen=True, order=True)
class LibcVersion:
    major: int
    minor: int


class Libc(Enum):
    value: str

    GLIBC = "glibc"
    MUSL = "musl"
    ANDROID = "android"

    def __str__(self) -> str:
        return self.value

    def get_current_version(self) -> LibcVersion:
        if self == Libc.MUSL:
            return _get_musl_version(_find_musl_libc())
        if self == Libc.GLIBC:
            return _get_glibc_version()
        msg = f"can't determine version of libc '{self}'"
        raise InvalidLibcError(msg)

    @staticmethod
    def detect() -> Libc:
        # check musl first, default to GLIBC
        try:
            _find_musl_libc()
            logger.debug("Detected musl libc")
            return Libc.MUSL  # noqa: TRY300
        except InvalidLibcError:
            logger.debug("Falling back to GNU libc")
            return Libc.GLIBC

    @property
    def tag_prefix(self) -> str:
        return {
            Libc.GLIBC: "manylinux",
            Libc.MUSL: "musllinux",
            Libc.ANDROID: "android",
        }[self]


def _find_musl_libc() -> Path:
    try:
        (dl_path,) = list(Path("/lib").glob("libc.musl-*.so.1"))
    except ValueError:
        msg = "musl libc not detected"
        logger.debug("%s", msg)
        raise InvalidLibcError(msg) from None

    return dl_path


def _get_musl_version(ld_path: Path) -> LibcVersion:
    try:
        ld = subprocess.run(
            [ld_path],
            check=False,
            errors="strict",
            stderr=subprocess.PIPE,
        ).stderr
    except FileNotFoundError as err:
        msg = "failed to determine musl version"
        logger.exception("%s", msg)
        raise InvalidLibcError(msg) from err

    match = re.search(r"Version (?P<major>\d+).(?P<minor>\d+).(?P<patch>\d+)", ld)
    if not match:
        msg = f"failed to parse musl version from string {ld!r}"
        raise InvalidLibcError(msg) from None

    return LibcVersion(int(match.group("major")), int(match.group("minor")))


def _get_glibc_version() -> LibcVersion:
    # CS_GNU_LIBC_VERSION is only for glibc and shall return e.g. "glibc 2.3.4"
    try:
        version_string: str | None = os.confstr("CS_GNU_LIBC_VERSION")
        if version_string is None:
            msg = "failed to determine glibc version"
            raise InvalidLibcError(msg)
        _, version = version_string.rsplit()
    except (AttributeError, OSError, ValueError) as err:
        # os.confstr() or CS_GNU_LIBC_VERSION not available (or a bad value)...
        msg = "failed to determine glibc version"
        raise InvalidLibcError(msg) from err

    m = re.match(r"(?P<major>[0-9]+)\.(?P<minor>[0-9]+)", version)
    if not m:
        msg = f"failed to parse glibc version from string {version!r}"
        raise InvalidLibcError(msg)

    return LibcVersion(int(m.group("major")), int(m.group("minor")))



from importlib import metadata
from pathlib import Path
from unittest.mock import call, patch

import pytest

from auditwheel._vendor.whichprovides import ProvidedBy
from auditwheel.sboms import create_sbom_for_wheel


def test_invalid_wheel_fname():
    with pytest.raises(ValueError, match="Failed to parse wheel file name"):
        create_sbom_for_wheel("not-a-wheel", [Path("path")])


@patch("auditwheel.sboms.whichprovides")
def test_create_sbom(whichprovides):
    whichprovides.return_value = {
        "path": ProvidedBy(
            package_type="deb",
            package_name="python3",
            package_version="3.10.6",
            distro="ubuntu",
        ),
    }

    auditwheel_version = metadata.version("auditwheel")
    wheel_fname = "testpackage-0.0.1-py3-none-any.whl"
    sbom = create_sbom_for_wheel(wheel_fname, [Path("path")])

    assert whichprovides.call_args_list == [call(["path"])]
    assert sbom == {
        "bomFormat": "CycloneDX",
        "specVersion": "1.4",
        "version": 1,
        "metadata": {
            "component": {
                "type": "library",
                "bom-ref": f"pkg:pypi/testpackage@0.0.1?file_name={wheel_fname}",
                "name": "testpackage",
                "version": "0.0.1",
                "purl": f"pkg:pypi/testpackage@0.0.1?file_name={wheel_fname}",
            },
            "tools": [{"name": "auditwheel", "version": auditwheel_version}],
        },
        "components": [
            {
                "type": "library",
                "bom-ref": f"pkg:pypi/testpackage@0.0.1?file_name={wheel_fname}",
                "name": "testpackage",
                "version": "0.0.1",
                "purl": f"pkg:pypi/testpackage@0.0.1?file_name={wheel_fname}",
            },
            {
                "type": "library",
                "bom-ref": "pkg:deb/ubuntu/python3@3.10.6#a0af9f865bf637e6736817f4ce552e4cdf7b8c36ea75bc254c1d1f0af744b5bf",  # noqa: E501
                "name": "python3",
                "version": "3.10.6",
                "purl": "pkg:deb/ubuntu/python3@3.10.6",
            },
        ],
        "dependencies": [
            {
                "ref": f"pkg:pypi/testpackage@0.0.1?file_name={wheel_fname}",
                "dependsOn": [
                    "pkg:deb/ubuntu/python3@3.10.6#a0af9f865bf637e6736817f4ce552e4cdf7b8c36ea75bc254c1d1f0af744b5bf",
                ],
            },
            {
                "ref": "pkg:deb/ubuntu/python3@3.10.6#a0af9f865bf637e6736817f4ce552e4cdf7b8c36ea75bc254c1d1f0af744b5bf",  # noqa: E501
            },
        ],
    }



from __future__ import annotations

import logging
from pathlib import Path
from typing import TYPE_CHECKING, Any

from auditwheel import options

if TYPE_CHECKING:
    import argparse

logger = logging.getLogger(__name__)


def configure_parser(sub_parsers: Any) -> None:  # noqa: ANN401
    help_ = "Audit a wheel for external shared library dependencies."
    p = sub_parsers.add_parser("show", help=help_, description=help_)
    p.add_argument("WHEEL_FILE", type=Path, help="Path to wheel file.")
    options.disable_isa_check(p)
    options.allow_pure_python_wheel(p)
    options.ldpaths(p)
    p.set_defaults(func=execute)


def printp(text: str) -> None:
    from textwrap import wrap

    print()
    print("\n".join(wrap(text, break_long_words=False, break_on_hyphens=False)))


def execute(args: argparse.Namespace, parser: argparse.ArgumentParser) -> int:
    from auditwheel import json
    from auditwheel.error import NonPlatformWheelError, WheelToolsError
    from auditwheel.wheel_abi import analyze_wheel_abi
    from auditwheel.wheeltools import get_wheel_architecture, get_wheel_libc

    wheel_file: Path = args.WHEEL_FILE
    fn = wheel_file.name

    if not wheel_file.is_file():
        parser.error(f"cannot access {wheel_file}. No such file")

    fn = wheel_file.name
    is_pure_python = False
    try:
        arch = get_wheel_architecture(fn)
    except (WheelToolsError, NonPlatformWheelError) as e:
        logger.warning("The architecture could not be deduced from the wheel filename")
        is_pure_python = isinstance(e, NonPlatformWheelError)
        arch = None

    try:
        libc = get_wheel_libc(fn)
    except WheelToolsError:
        logger.debug("The libc could not be deduced from the wheel filename")
        libc = None

    try:
        winfo = analyze_wheel_abi(
            libc,
            arch,
            wheel_file,
            frozenset(),
            disable_isa_ext_check=args.DISABLE_ISA_EXT_CHECK,
            allow_graft=False,
            args_ldpaths=args.LDPATHS,
        )
    except NonPlatformWheelError as e:
        logger.info("%s", e.message)
        if is_pure_python and args.ALLOW_PURE_PY_WHEEL:
            return 0
        return 1

    policies = winfo.policies

    libs_with_versions = [f"{k} with versions {v}" for k, v in winfo.versioned_symbols.items()]

    printp(
        f'{fn} is consistent with the following platform tag: "{winfo.overall_policy.name}".',
    )

    if winfo.pyfpe_policy == policies.linux:
        printp(
            "This wheel uses the PyFPE_jbuf function, which is not compatible with the"
            " manylinux/musllinux tags. (see https://www.python.org/dev/peps/pep-0513/"
            "#fpectl-builds-vs-no-fpectl-builds)",
        )
        if args.verbose < 1:
            return 0

    if winfo.ucs_policy == policies.linux:
        printp(
            "This wheel is compiled against a narrow unicode (UCS2) "
            "version of Python, which is not compatible with the "
            "manylinux/musllinux tags.",
        )
        if args.verbose < 1:
            return 0

    if winfo.machine_policy == policies.linux:
        printp("This wheel depends on unsupported ISA extensions.")
        if args.verbose < 1:
            return 0

    if len(libs_with_versions) == 0:
        printp(
            "The wheel references no external versioned symbols from "
            "system-provided shared libraries.",
        )
    else:
        printp(
            "The wheel references external versioned symbols in these "
            f"system-provided shared libraries: {', '.join(libs_with_versions)}",
        )

    if winfo.sym_policy < policies.highest:
        printp(
            f'This constrains the platform tag to "{winfo.sym_policy.name}". '
            "In order to achieve a more compatible tag, you would "
            "need to recompile a new wheel from source on a system "
            "with earlier versions of these libraries, such as "
            "a recent manylinux image.",
        )
        if args.verbose < 1:
            return 0

    libs = winfo.external_refs[policies.lowest.name].libs
    if len(libs) == 0:
        printp("The wheel requires no external shared libraries! :)")
    else:
        printp("The following external shared libraries are required by the wheel:")
        print(json.dumps(dict(sorted(libs.items()))))

    for p in policies:
        if p > winfo.overall_policy:
            libs = winfo.external_refs[p.name].libs
            if len(libs):
                printp(
                    f"In order to achieve the tag platform tag {p.name!r} "
                    "the following shared library dependencies "
                    "will need to be eliminated:",
                )
                printp(", ".join(sorted(libs.keys())))
            blacklist = winfo.external_refs[p.name].blacklist
            if len(blacklist):
                printp(
                    f"In order to achieve the tag platform tag {p.name!r} "
                    "the following black-listed symbol dependencies "
                    "will need to be eliminated:",
                )
                for key in sorted(blacklist.keys()):
                    printp(f"From {key}: " + ", ".join(sorted(blacklist[key])))
    return 0



# /// script
# dependencies = ["nox>=2025.2.9"]
# ///

from __future__ import annotations

import os
import platform
import sys
from pathlib import Path

import nox

nox.needs_version = ">=2025.2.9"

PYTHON_ALL_VERSIONS = ["3.10", "3.11", "3.12", "3.13", "3.14"]
RUNNING_CI = "TRAVIS" in os.environ or "GITHUB_ACTIONS" in os.environ

wheel = ""
sdist = ""


@nox.session(reuse_venv=True)
def lint(session: nox.Session) -> None:
    """Run linters on the codebase."""
    session.install("pre-commit")
    session.run("pre-commit", "run", "--all-files")


@nox.session(default=False)
def coverage(session: nox.Session) -> None:
    """Run coverage using unit tests."""
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, "coverage")
    session.install("-e", ".", *deps)
    session.run(
        "python",
        "-m",
        "pytest",
        "tests/unit",
        "--cov=auditwheel",
        "--cov-report=term-missing",
    )


def _docker_images(session: nox.Session) -> list[str]:
    tmp_dir = Path(session.create_tmp())
    script = tmp_dir / "list_images.py"
    images_file = tmp_dir / "images.lst"
    script.write_text(
        rf"""
import sys
from pathlib import Path
sys.path.append("./tests/integration")
from test_manylinux import MANYLINUX_IMAGES
images = "\n".join(MANYLINUX_IMAGES.values())
Path(r"{images_file}").write_text(images)
""",
    )
    session.run("python", str(script), silent=True)
    return images_file.read_text().splitlines()


def _download_wheels_for_tests(session: nox.Session) -> None:
    wheels = [
        # for tests/integration/test_bundled_wheels.py::test_analyze_wheel_abi_static_exe
        ("patchelf==0.17.2.1", "py38", "manylinux1_x86_64"),
        # for tests/integration/test_bundled_wheels.py::test_weak_symbols_not_blacklisted
        ("cryptography==46.0.3", "cp38", "manylinux_2_17_x86_64"),
    ]
    for package, python_tag, platform_tag in wheels:
        session.run(
            "pip",
            "download",
            "--only-binary=:all:",
            "--no-deps",
            "--dest=./tests/integration/",
            f"--platform={platform_tag}",
            f"--implementation={python_tag[:2]}",
            f"--python-version={python_tag[2:]}",
            package,
        )


@nox.session(python=PYTHON_ALL_VERSIONS, default=False)
def tests(session: nox.Session) -> None:
    """Run tests."""
    posargs = session.posargs
    dep_group = "coverage" if RUNNING_CI else "test"
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, dep_group)
    session.install("-U", "pip")
    session.install("-e", ".", *deps)
    _download_wheels_for_tests(session)

    if RUNNING_CI:
        posargs.extend(["--cov", "auditwheel", "--cov-config", "pyproject.toml"])
        if sys.platform != "darwin":
            # pull manylinux images that will be used.
            # this helps passing tests which would otherwise timeout.
            for image in _docker_images(session):
                session.run("docker", "pull", image, external=True)

    session.run("pytest", "-s", *posargs)
    if RUNNING_CI:
        machine = os.environ.get("AUDITWHEEL_ARCH", platform.machine()).lower()
        suffix = f"{session.python}-{sys.platform}-{machine}"
        session.run("coverage", "xml", f"-ocoverage-{suffix}.xml")


@nox.session(python=["3.10"], default=False)
def build(session: nox.Session) -> None:
    session.install("build")
    tmp_dir = Path(session.create_tmp()) / "build-output"
    session.run("python", "-m", "build", "--outdir", str(tmp_dir))
    (wheel_path,) = tmp_dir.glob("*.whl")
    (sdist_path,) = tmp_dir.glob("*.tar.gz")
    Path("dist").mkdir(exist_ok=True)
    wheel_path.rename(f"dist/{wheel_path.name}")
    sdist_path.rename(f"dist/{sdist_path.name}")

    global sdist  # noqa: PLW0603
    sdist = f"dist/{sdist_path.name}"
    global wheel  # noqa: PLW0603
    wheel = f"dist/{wheel_path.name}"


def _test_dist(session: nox.Session, path: str) -> None:
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, "test")
    session.install(path, *deps)
    session.run("pytest", "tests/unit")


@nox.session(name="test-sdist", python=PYTHON_ALL_VERSIONS, requires=["build"])
def test_sdist(session: nox.Session) -> None:
    """Do not run explicitly."""
    _test_dist(session, sdist)


@nox.session(name="test-wheel", python=PYTHON_ALL_VERSIONS, requires=["build"])
def test_wheel(session: nox.Session) -> None:
    """Do not run explicitly."""
    _test_dist(session, wheel)


@nox.session(python=PYTHON_ALL_VERSIONS, reuse_venv=True, default=False)
def develop(session: nox.Session) -> None:
    session.run("python", "-m", "pip", "install", "--upgrade", "pip")
    pyproject = nox.project.load_toml("pyproject.toml")
    deps = nox.project.dependency_groups(pyproject, "dev")
    session.install("-e", ".", *deps)
    _download_wheels_for_tests(session)


if __name__ == "__main__":
    nox.main()



"""General tools for working with wheels

Tools that aren't specific to delocation
"""

from __future__ import annotations

import csv
import hashlib
import logging
import os
import re
import zlib
from base64 import urlsafe_b64encode
from datetime import datetime, timezone
from itertools import product
from pathlib import Path
from typing import TYPE_CHECKING

from packaging.utils import parse_wheel_filename

from auditwheel._vendor.wheel.pkginfo import read_pkg_info, write_pkg_info
from auditwheel.architecture import Architecture
from auditwheel.error import NonPlatformWheelError, WheelToolsError
from auditwheel.libc import Libc
from auditwheel.tmpdirs import InTemporaryDirectory
from auditwheel.tools import dir2zip, unique_by_index, walk, zip2dir

if TYPE_CHECKING:
    from collections.abc import Generator, Iterable
    from types import TracebackType

logger = logging.getLogger(__name__)


# Copied from wheel 0.31.1
WHEEL_INFO_RE = re.compile(
    r"""^(?P<namever>(?P<name>.+?)-(?P<ver>\d.*?))(-(?P<build>\d.*?))?
     -(?P<pyver>[a-z].+?)-(?P<abi>.+?)-(?P<plat>.+?)(\.whl|\.dist-info)$""",
    re.VERBOSE,
).match


def _dist_info_dir(bdist_dir: Path) -> Path:
    """Get the .dist-info directory from an unpacked wheel

    Parameters
    ----------
    bdist_dir : Path
        Path of unpacked wheel file
    """

    info_dirs = list(bdist_dir.glob("*.dist-info"))
    if len(info_dirs) != 1:
        msg = "Should be exactly one `*.dist_info` directory"
        raise WheelToolsError(msg)
    return info_dirs[0]


def rewrite_record(bdist_dir: Path) -> None:
    """Rewrite RECORD file with hashes for all files in `wheel_sdir`

    Copied from :method:`wheel.bdist_wheel.bdist_wheel.write_record`

    Will also unsign wheel

    Parameters
    ----------
    bdist_dir : Path
        Path of unpacked wheel file
    """
    info_dir = _dist_info_dir(bdist_dir)
    record_path = info_dir / "RECORD"
    record_relpath = record_path.relative_to(bdist_dir)
    # Unsign wheel - because we're invalidating the record hash
    sig_path = info_dir / "RECORD.jws"
    if sig_path.exists():
        sig_path.unlink()

    def files() -> Generator[Path]:
        for dir_, _, files in walk(bdist_dir):
            for file in files:
                yield dir_ / file

    def skip(path: Path) -> bool:
        """Wheel hashes every possible file."""
        return path == record_relpath

    with record_path.open("w+", newline="", encoding="utf-8") as record_file:
        writer = csv.writer(record_file)
        for path in files():
            relative_path = path.relative_to(bdist_dir)
            if skip(relative_path):
                hash_ = ""
                size = ""
            else:
                data = path.read_bytes()
                digest = hashlib.sha256(data).digest()
                sha256 = urlsafe_b64encode(digest).rstrip(b"=").decode("ascii")
                hash_ = f"sha256={sha256}"
                size = f"{len(data)}"
            record_path_ = path.relative_to(bdist_dir).as_posix()
            writer.writerow((record_path_, hash_, size))


class InWheel(InTemporaryDirectory):
    """Context manager for doing things inside wheels

    On entering, you'll find yourself in the root tree of the wheel.  If you've
    asked for an output wheel, then on exit we'll rewrite the wheel record and
    pack stuff up for you.
    """

    def __init__(self, in_wheel: Path, out_wheel: Path | None = None) -> None:
        """Initialize in-wheel context manager

        Parameters
        ----------
        in_wheel : Path
            filename of wheel to unpack and work inside
        out_wheel : None or Path:
            filename of wheel to write after exiting.  If None, don't write and
            discard
        """
        self.in_wheel = in_wheel.absolute()
        self.out_wheel = None if out_wheel is None else out_wheel.absolute()
        self.zip_compression_level = zlib.Z_DEFAULT_COMPRESSION
        super().__init__()

    def __enter__(self) -> Path:
        zip2dir(self.in_wheel, self.name)
        return super().__enter__()

    def __exit__(
        self,
        exc: type[BaseException] | None,
        value: BaseException | None,
        tb: TracebackType | None,
    ) -> None:
        if self.out_wheel is not None:
            rewrite_record(self.name)
            date_time = None
            timestamp = os.environ.get("SOURCE_DATE_EPOCH")
            if timestamp:
                date_time = datetime.fromtimestamp(int(timestamp), tz=timezone.utc)
            dir2zip(self.name, self.out_wheel, self.zip_compression_level, date_time)
        return super().__exit__(exc, value, tb)


class InWheelCtx(InWheel):
    """Context manager for doing things inside wheels

    On entering, you'll find yourself in the root tree of the wheel.  If you've
    asked for an output wheel, then on exit we'll rewrite the wheel record and
    pack stuff up for you.

    The context manager returns itself from the __enter__ method, so you can
    set things like ``out_wheel``.  This is useful when processing in the wheel
    will dicate what the output wheel name is, or whether you want to save at
    all.

    The current path of the wheel contents is set in the attribute
    ``wheel_path``.
    """

    def __init__(self, in_wheel: Path, out_wheel: Path | None = None) -> None:
        """Init in-wheel context manager returning self from enter

        Parameters
        ----------
        in_wheel : Path
            filename of wheel to unpack and work inside
        out_wheel : None or Path:
            filename of wheel to write after exiting.  If None, don't write and
            discard
        """
        super().__init__(in_wheel, out_wheel)
        self.path: Path | None = None

    def __enter__(self):  # type: ignore[no-untyped-def]
        self.path = super().__enter__()
        return self

    def iter_files(self) -> Generator[Path]:
        if self.path is None:
            msg = "This function should be called from context manager"
            raise ValueError(msg)
        record_name = _dist_info_dir(self.path) / "RECORD"
        record = record_name.read_text()
        reader = csv.reader(r for r in record.splitlines())
        for row in reader:
            filename = row[0]
            yield Path(filename)


def add_platforms(
    wheel_ctx: InWheelCtx,
    platforms: list[str],
    remove_platforms: Iterable[str] = (),
) -> Path:
    """Add platform tags `platforms` to a wheel

    Add any platform tags in `platforms` that are missing
    to wheel_ctx's filename and ``WHEEL`` file.

    Parameters
    ----------
    wheel_ctx : InWheelCtx
        An open wheel context
    platforms : list
        platform tags to add to wheel filename and WHEEL tags - e.g.
        ``('macosx_10_9_intel', 'macosx_10_9_x86_64')
    remove_platforms : iterable
        platform tags to remove to the wheel filename and WHEEL tags, e.g.
        ``('linux_x86_64',)`` when ``('manylinux_x86_64')`` is added
    """
    if wheel_ctx.path is None:
        msg = "This function should be called from wheel_ctx context manager"
        raise ValueError(msg)

    to_remove = list(remove_platforms)  # we might want to modify this, make a copy

    definitely_not_purelib = False

    info_fname = _dist_info_dir(wheel_ctx.path) / "WHEEL"
    info = read_pkg_info(info_fname)
    # Check what tags we have
    if wheel_ctx.out_wheel is not None:
        out_dir = wheel_ctx.out_wheel.parent
        wheel_fname = wheel_ctx.out_wheel.name
    else:
        out_dir = Path.cwd()
        wheel_fname = wheel_ctx.in_wheel.name

    original_fname_tags = get_wheel_platforms(wheel_fname)
    logger.info("Previous filename tags: %s", ", ".join(original_fname_tags))
    fname_tags = [tag for tag in original_fname_tags if tag not in to_remove]
    fname_tags = unique_by_index(fname_tags + platforms)

    # Can't be 'any' and another platform
    if "any" in fname_tags and len(fname_tags) > 1:
        fname_tags.remove("any")
        to_remove.append("any")
        definitely_not_purelib = True

    if fname_tags != original_fname_tags:
        logger.info("New filename tags: %s", ", ".join(fname_tags))
    else:
        logger.info("No filename tags change needed.")

    fparts = {
        "prefix": wheel_fname.rsplit("-", maxsplit=1)[0],
        "plat": ".".join(sorted(fname_tags)),
        "ext": Path(wheel_fname).suffix,
    }
    out_wheel_fname = "{prefix}-{plat}{ext}".format(**fparts)
    out_wheel = out_dir / out_wheel_fname

    in_info_tags = [tag for name, tag in info.items() if name == "Tag"]
    logger.info("Previous WHEEL info tags: %s", ", ".join(in_info_tags))
    # Python version, C-API version combinations
    pyc_apis = ["-".join(tag.split("-")[:2]) for tag in in_info_tags]
    # unique Python version, C-API version combinations
    pyc_apis = unique_by_index(pyc_apis)
    # Add new platform tags for each Python version, C-API combination
    wanted_tags = ["-".join(tup) for tup in product(pyc_apis, platforms)]
    new_tags = [tag for tag in wanted_tags if tag not in in_info_tags]
    unwanted_tags = ["-".join(tup) for tup in product(pyc_apis, to_remove)]
    updated_tags = [tag for tag in in_info_tags if tag not in unwanted_tags]
    updated_tags += new_tags
    if updated_tags != in_info_tags:
        del info["Tag"]
        for tag in updated_tags:
            info.add_header("Tag", tag)

        if definitely_not_purelib:
            del info["Root-Is-Purelib"]
            info["Root-Is-Purelib"] = "False"
            logger.info("Changed wheel type to Platlib")

        logger.info("New WHEEL info tags: %s", ", ".join(info.get_all("Tag")))
        write_pkg_info(info_fname, info)
    else:
        logger.info("No WHEEL info change needed.")
    wheel_ctx.out_wheel = out_wheel
    return out_wheel


def get_wheel_architecture(filename: str) -> Architecture:
    result: set[Architecture] = set()
    missed = False
    pure = True
    for platform in get_wheel_platforms(filename):
        found = False
        pure_ = platform == "any"
        pure = pure and pure_
        missed = missed or pure_
        if not pure_:
            for arch in Architecture:
                if platform.endswith(f"_{arch.value}"):
                    result.add(arch.baseline)
                    found = True
            if not found:
                logger.warning(
                    "couldn't guess architecture for platform tag '%s'",
                    platform,
                )
                missed = True
    if len(result) == 0:
        if pure:
            raise NonPlatformWheelError(None, None)
        msg = "unknown architecture"
        raise WheelToolsError(msg)
    if missed or len(result) > 1:
        if len(result) == 1:
            msg = "wheels with multiple architectures are not supported"
        else:
            msg = f"wheels with multiple architectures are not supported, got {result}"
        raise WheelToolsError(msg)
    return result.pop()


def get_wheel_libc(filename: str) -> Libc:
    result: set[Libc] = set()
    for platform in get_wheel_platforms(filename):
        for libc in Libc:
            if platform.startswith(libc.tag_prefix):
                result.add(libc)
    if len(result) == 0:
        msg = "unknown libc used"
        raise WheelToolsError(msg)
    if len(result) > 1:
        msg = f"wheels with multiple libc are not supported, got {result}"
        raise WheelToolsError(msg)
    return result.pop()


def get_wheel_platforms(filename: str) -> list[str]:
    _, _, _, in_tags = parse_wheel_filename(filename)
    return sorted({tag.platform for tag in in_tags})


def android_api_level(tag: str) -> int:
    match = re.match(r"android_(\d+)", tag)
    if match is None:
        msg = f"invalid tag: {tag}"
        raise ValueError(msg)
    return int(match[1])



from __future__ import annotations

import re
import shutil
import tempfile
import zipfile
from pathlib import Path

import pytest

from auditwheel._vendor.wheel.pkginfo import read_pkg_info
from auditwheel.architecture import Architecture
from auditwheel.error import NonPlatformWheelError
from auditwheel.libc import Libc
from auditwheel.wheeltools import (
    InWheelCtx,
    WheelToolsError,
    add_platforms,
    android_api_level,
    get_wheel_architecture,
    get_wheel_libc,
    get_wheel_platforms,
)

HERE = Path(__file__).parent.resolve()


@pytest.mark.parametrize(
    ("filename", "expected"),
    [(f"foo-1.0-py3-none-linux_{arch}.whl", arch) for arch in Architecture]
    + [
        ("foo-1.0-py3-none-linux_x86_64.manylinux1_x86_64.whl", Architecture.x86_64),
        ("foo-1.0-py3-none-android_21_x86_64.whl", Architecture.x86_64),
        ("foo-1.0-py3-none-android_21_arm64_v8a.whl", Architecture.aarch64),
    ],
)
def test_get_wheel_architecture(filename: str, expected: Architecture) -> None:
    arch = get_wheel_architecture(filename)
    assert arch == expected.baseline


def test_get_wheel_architecture_unknown() -> None:
    with pytest.raises(WheelToolsError, match=re.escape("unknown architecture")):
        get_wheel_architecture("foo-1.0-py3-none-linux_mipsel.whl")


def test_get_wheel_architecture_pure() -> None:
    with pytest.raises(NonPlatformWheelError):
        get_wheel_architecture("foo-1.0-py3-none-any.whl")


@pytest.mark.parametrize(
    "filename",
    [
        "foo-1.0-py3-none-linux_x86_64.linux_aarch64.whl",
        "foo-1.0-py3-none-linux_x86_64.linux_mipsel.whl",
        "foo-1.0-py3-none-linux_x86_64.any.whl",
    ],
)
def test_get_wheel_architecture_multiple(filename: str) -> None:
    match = re.escape("multiple architectures are not supported")
    with pytest.raises(WheelToolsError, match=match):
        get_wheel_architecture(filename)


@pytest.mark.parametrize(
    ("filename", "expected"),
    [
        ("foo-1.0-py3-none-manylinux1_x86_64.whl", Libc.GLIBC),
        ("foo-1.0-py3-none-manylinux1_x86_64.manylinux2010_x86_64.whl", Libc.GLIBC),
        ("foo-1.0-py3-none-musllinux_1_1_x86_64.whl", Libc.MUSL),
        ("foo-1.0-py3-none-android_24_arm64_v8a.whl", Libc.ANDROID),
    ],
)
def test_get_wheel_libc(filename: str, expected: Libc) -> None:
    libc = get_wheel_libc(filename)
    assert libc == expected


@pytest.mark.parametrize(
    "filename",
    ["foo-1.0-py3-none-any.whl", "foo-1.0-py3-none-something.whl"],
)
def test_get_wheel_libc_unknown(filename: str) -> None:
    with pytest.raises(WheelToolsError, match=re.escape("unknown libc used")):
        get_wheel_libc(filename)


@pytest.mark.parametrize(
    "filename",
    ["foo-1.0-py3-none-manylinux1_x86_64.musllinux_1_1_x86_64.whl"],
)
def test_get_wheel_libc_multiple(filename: str) -> None:
    match = re.escape("multiple libc are not supported")
    with pytest.raises(WheelToolsError, match=match):
        get_wheel_libc(filename)


@pytest.mark.parametrize(
    ("filename", "expected"),
    [
        ("foo-1.0-py3-none-manylinux1_x86_64.whl", ["manylinux1_x86_64"]),
        ("foo-1.0-py3-none-any.any.whl", ["any"]),
        ("foo-1.0-py3-none-linux_x86_64.any.whl", ["any", "linux_x86_64"]),
        ("foo-1.0-py3-none-any.linux_x86_64.whl", ["any", "linux_x86_64"]),
        ("foo-1.0-py2.py3-none-linux_x86_64.any.whl", ["any", "linux_x86_64"]),
    ],
)
def test_get_wheel_platforms(filename: str, expected: list[str]) -> None:
    assert get_wheel_platforms(filename) == expected


@pytest.mark.parametrize(
    ("tag", "expected"),
    [
        ("android_21_arm64_v8a", 21),
        ("android_21_x86_64", 21),
        ("android_9_whatever", 9),
        ("linux_x86_64", None),
    ],
)
def test_android_api_level(tag, expected):
    if expected is None:
        with pytest.raises(ValueError, match=f"invalid tag: {tag}"):
            android_api_level(tag)
    else:
        assert android_api_level(tag) == expected


def test_inwheel_tmpdir(tmp_path, monkeypatch):
    wheel_path = (
        HERE / "../integration/arch-wheels/glibc/testsimple-0.0.1-cp313-cp313-linux_x86_64.whl"
    )
    tmp_path = tmp_path.resolve(strict=True)
    tmpdir = tmp_path / "tmpdir"
    tmpdir.mkdir()
    tmpdir_symlink = tmp_path / "symlink"
    tmpdir_symlink.symlink_to(str(tmpdir), target_is_directory=True)
    monkeypatch.setattr(tempfile, "gettempdir", lambda: str(tmpdir_symlink))
    with InWheelCtx(wheel_path, tmp_path / wheel_path.name) as context:
        Path(context._tmpdir.name).relative_to(tmpdir_symlink)
        context.name.relative_to(tmpdir)


def test_inwheel_no_manager(tmp_path):
    wheel_path = (
        HERE / "../integration/arch-wheels/glibc/testsimple-0.0.1-cp313-cp313-linux_x86_64.whl"
    )
    context = InWheelCtx(wheel_path, tmp_path / wheel_path.name)
    with pytest.raises(
        ValueError,
        match=re.escape("This function should be called from context manager"),
    ):
        next(context.iter_files())
    with pytest.raises(
        ValueError,
        match=re.escape("This function should be called from wheel_ctx context manager"),
    ):
        add_platforms(context, [], [])


def test_add_platforms_no_duplicate_root_is_purelib(tmp_path):
    """Regression test for #642: add_platforms should not duplicate Root-Is-Purelib."""
    wheel_name = "testpkg-0.0.1-py3-none-any.whl"
    wheel_path = tmp_path / wheel_name
    dist_info = "testpkg-0.0.1.dist-info"

    # Create a minimal wheel with Root-Is-Purelib already set to false
    with zipfile.ZipFile(wheel_path, "w") as zf:
        zf.writestr(
            f"{dist_info}/WHEEL",
            "Wheel-Version: 1.0\nGenerator: test\nRoot-Is-Purelib: false\nTag: py3-none-any\n",
        )
        zf.writestr(
            f"{dist_info}/METADATA",
            "Metadata-Version: 2.1\nName: testpkg\nVersion: 0.0.1\n",
        )
        zf.writestr(f"{dist_info}/RECORD", "")

    out_wheel_path = tmp_path / "out" / wheel_name
    out_wheel_path.parent.mkdir()

    with InWheelCtx(wheel_path, out_wheel_path) as ctx:
        add_platforms(ctx, ["manylinux_2_35_x86_64"])

    # Find the output wheel (name will have changed)
    out_wheels = list((tmp_path / "out").glob("*.whl"))
    assert len(out_wheels) == 1

    with zipfile.ZipFile(out_wheels[0]) as zf:
        wheel_info = next(n for n in zf.namelist() if n.endswith("/WHEEL"))
        info = read_pkg_info(zf.extract(wheel_info, tmp_path / "extracted"))

    purelib_values = info.get_all("Root-Is-Purelib")
    assert purelib_values is not None
    assert len(purelib_values) == 1, (
        f"Expected exactly one Root-Is-Purelib entry, got {len(purelib_values)}"
    )


def test_inwheel_no_distinfo():
    wheel_path = (
        HERE / "../integration/arch-wheels/glibc/testsimple-0.0.1-cp313-cp313-linux_x86_64.whl"
    )
    with InWheelCtx(wheel_path, None) as context:
        dist_info = list(context.path.glob("*.dist-info"))
        assert len(dist_info) == 1
        shutil.rmtree(dist_info[0])
        with pytest.raises(
            WheelToolsError,
            match=re.escape("Should be exactly one `*.dist_info` directory"),
        ):
            next(context.iter_files())



from __future__ import annotations

from typing import TYPE_CHECKING

from auditwheel.condatools import InCondaPkgCtx
from auditwheel.wheeltools import InWheelCtx

if TYPE_CHECKING:
    from pathlib import Path


def InGenericPkgCtx(  # noqa: N802
    in_path: Path,
    out_path: Path | None = None,
) -> InWheelCtx | InCondaPkgCtx:
    """Factory that returns a InWheelCtx or InCondaPkgCtx
    context manager depending on the file extension
    """
    if in_path.name.endswith(".whl"):
        return InWheelCtx(in_path, out_path)
    if in_path.name.endswith(".tar.bz2"):
        if out_path is not None:
            raise NotImplementedError
        return InCondaPkgCtx(in_path)
    msg = f"Invalid package: {in_path}. File formats supported: .whl, .tar.bz2"
    raise ValueError(msg)
